@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$*/
+
 /* Plane extraction device */
 #include "gx.h"
 #include "gserrors.h"
@@ -112,7 +114,36 @@ static const gx_device_plane_extract gs_plane_extract_device = {
         NULL,
         gx_no_create_compositor, /* WRONG */
         NULL,
-        gx_default_text_begin
+        gx_default_text_begin,
+        gx_default_finish_copydevice,
+        NULL,	/* deprecated and never implemented transparency procs */
+        NULL,	/*	|	*/
+        NULL,	/*	|	*/
+        NULL,	/*	|	*/
+        NULL,	/*	V	*/
+        NULL,	/* DeviceN support color mapping procs */
+        NULL,	/*	|	*/
+        NULL,	/*	|	*/
+        NULL,	/*	|	*/
+        NULL,	/* pattern_manage */
+        gx_default_fill_rectangle_hl_color,
+        gx_default_include_color_space,
+        gx_default_fill_linear_color_scanline,
+        gx_default_fill_linear_color_trapezoid,
+        gx_default_fill_linear_color_triangle,
+        gx_default_update_spot_equivalent_colors,
+        gx_default_ret_devn_params,
+        gx_default_fillpage,
+        NULL,	/* push_transparency_state */
+        NULL,	/* pop_transparency_state */
+        NULL,	/* put_image */
+        NULL,	/* dev_spec_op */
+        NULL,	/* copy_planes */
+        NULL,	/* get_profile */
+        NULL,	/* set_graphics_type_tag */
+        gx_default_strip_copy_rop2,
+        gx_default_strip_tile_rect_devn,
+        gx_default_copy_alpha_hl_color
     },
     /* device-specific members */
     NULL,				/* target */
@@ -860,7 +891,8 @@ plane_cmap_rgb(frac r, frac g, frac b, gx_device_color * pdc,
 }
 static void
 plane_cmap_cmyk(frac c, frac m, frac y, frac k, gx_device_color * pdc,
-    const gs_imager_state *pis_image, gx_device *dev, gs_color_select_t select)
+    const gs_imager_state *pis_image, gx_device *dev, gs_color_select_t select,
+    const gs_color_space *source_pcs)
 {
     const plane_image_enum_t *ppie =
         (const plane_image_enum_t *)pis_image->client_data;
@@ -870,7 +902,7 @@ plane_cmap_cmyk(frac c, frac m, frac y, frac k, gx_device_color * pdc,
     gx_device_color dcolor;
 
     gx_remap_concrete_cmyk(c, m, y, k, &dcolor, ppie->pis,
-                           (gx_device *)edev, select);
+                           (gx_device *)edev, select, NULL);
     reduce_drawing_color(pdc, edev, &dcolor, &lop);
 }
 static void
