@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /*
  * Microsoft Windows platform support for Ghostscript.
  *
@@ -27,9 +29,13 @@
 /* DLL version must now be used under MS-Windows */
 /* Russell Lang 16 March 1996 */
 
+/* prevent gp.h from defining fopen */
+#define fopen fopen
+
 #include "stdio_.h"
 #include "string_.h"
 #include "memory_.h"
+#include "stat_.h"
 #include "pipe_.h"
 #include <stdlib.h>
 #include <stdarg.h>
@@ -39,17 +45,20 @@
 #include <fcntl.h>
 #include <signal.h>
 #include "gx.h"
+
 #include "gp.h"
 #include "gpcheck.h"
 #include "gpmisc.h"
 #include "gserrors.h"
 #include "gsexit.h"
+#include "scommon.h"
 
 #include "windows_.h"
 #include <stdarg.h>
 #include <shellapi.h>
 #include <winspool.h>
 #include "gp_mswin.h"
+#include "winrtsup.h"
 
 /* Library routines not declared in a standard header */
 extern char *getenv(const char *);
@@ -65,7 +74,9 @@ HINSTANCE phInstance;
 BOOL is_win32s = FALSE;
 
 const LPSTR szAppName = "Ghostscript";
+#ifndef METRO
 static int is_printer(const char *name);
+#endif
 
 /* ====== Generic platform procedures ====== */
 
@@ -110,7 +121,9 @@ int gp_cache_query(int type, byte* key, int keylen, void **buffer,
 /* ------ Printer accessing ------ */
 
 /* Forward references */
+#ifndef METRO
 static int gp_printfile(const char *, const char *);
+#endif
 
 /* Open a connection to a printer.  A null file name means use the */
 /* standard printer connected to the machine, if any. */
@@ -120,6 +133,9 @@ gp_open_printer(const gs_memory_t *mem,
                       char         fname[gp_file_name_sizeof],
                       int          binary_mode)
 {
+#ifdef METRO
+    return gp_fopen(fname, (binary_mode ? "wb" : "w"));
+#else
     if (is_printer(fname)) {
         FILE *pfile;
 
@@ -134,20 +150,26 @@ gp_open_printer(const gs_memory_t *mem,
         return NULL;	/* not supported, use %printer%name instead  */
     else
         return gp_fopen(fname, (binary_mode ? "wb" : "w"));
+#endif
 }
 
 /* Close the connection to the printer. */
 void
 gp_close_printer(const gs_memory_t *mem, FILE * pfile, const char *fname)
 {
+#ifdef METRO
+    fclose(pfile);
+#else
     fclose(pfile);
     if (!is_printer(fname))
         return;			/* a file, not a printer */
 
     gp_printfile(win_prntmp, fname);
     unlink(win_prntmp);
+#endif
 }
 
+#ifndef METRO
 /* Dialog box to select printer port */
 DLGRETURN CALLBACK
 SpoolDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
@@ -221,7 +243,9 @@ is_printer(const char *name)
 /* This is messy because of past support for old version of Windows. */
 
 /* Win95, WinNT: Use OpenPrinter, WritePrinter etc. */
+#ifndef METRO
 static int gp_printfile_win32(const char *filename, char *port);
+#endif
 
 /*
  * Valid values for pmport are:
@@ -244,7 +268,7 @@ gp_printfile(const char *filename, const char *pmport)
 
         /* WinNT stores default printer in registry and win.ini */
         /* Win95 stores default printer in win.ini */
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         GetProfileString("windows", "device", "", buf, sizeof(buf));
 #else
         wchar_t wbuf[512];
@@ -298,7 +322,7 @@ get_queues(void)
         char buf[256];
 
         free(enumbuffer);
-        sprintf(buf, "EnumPrinters() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "EnumPrinters() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         return NULL;
     }
@@ -356,10 +380,14 @@ get_queuename(char *portname, const char *queue)
     free(buffer);
     return TRUE;
 }
+#endif
 
 BOOL gp_OpenPrinter(char *port, LPHANDLE printer)
 {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef METRO
+    return FALSE;
+#else
+#ifdef GS_NO_UTF8
     return OpenPrinter(port, printer, NULL);
 #else
     BOOL opened;
@@ -370,8 +398,10 @@ BOOL gp_OpenPrinter(char *port, LPHANDLE printer)
     free(uni);
     return opened;
 #endif
+#endif
 }
 
+#ifndef METRO
 /* True Win32 method, using OpenPrinter, WritePrinter etc. */
 static int
 gp_printfile_win32(const char *filename, char *port)
@@ -399,7 +429,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!gp_OpenPrinter(port, &printer)) {
         char buf[256];
 
-        sprintf(buf, "OpenPrinter() failed for \042%s\042, error code = %d", port, GetLastError());
+        gs_sprintf(buf, "OpenPrinter() failed for \042%s\042, error code = %d", port, GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         free(buffer);
         return FALSE;
@@ -412,7 +442,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!StartDocPrinter(printer, 1, (LPBYTE) & di)) {
         char buf[256];
 
-        sprintf(buf, "StartDocPrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "StartDocPrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         AbortPrinter(printer);
         free(buffer);
@@ -433,7 +463,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!EndDocPrinter(printer)) {
         char buf[256];
 
-        sprintf(buf, "EndDocPrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "EndDocPrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         AbortPrinter(printer);
         return FALSE;
@@ -441,7 +471,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!ClosePrinter(printer)) {
         char buf[256];
 
-        sprintf(buf, "ClosePrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "ClosePrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         return FALSE;
     }
@@ -456,7 +486,7 @@ gp_printfile_win32(const char *filename, char *port)
 FILE *mswin_popen(const char *cmd, const char *mode)
 {
     SECURITY_ATTRIBUTES saAttr;
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     STARTUPINFO siStartInfo;
 #else
     STARTUPINFOW siStartInfo;
@@ -469,7 +499,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     HANDLE hChildStderrWr = INVALID_HANDLE_VALUE;
     HANDLE hProcess = GetCurrentProcess();
     int handle = 0;
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     char *command = NULL;
 #else
     wchar_t *command = NULL;
@@ -519,7 +549,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     siStartInfo.hStdError = hChildStderrWr;
 
     if (handle == 0) {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         command = (char *)malloc(strlen(cmd)+1);
         if (command)
             strcpy(command, cmd);
@@ -533,7 +563,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     }
 
     if (handle == 0)
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         if (!CreateProcess(NULL,
 #else
         if (!CreateProcessW(NULL,
@@ -576,6 +606,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     }
     return pipe;
 }
+#endif
 
 /* ------ File naming and accessing ------ */
 
@@ -594,17 +625,35 @@ gp_open_scratch_file(const gs_memory_t *mem,
     FILE *f = NULL;
     char sTempDir[_MAX_PATH];
     char sTempFileName[_MAX_PATH];
+#if defined(METRO) || !defined(GS_NO_UTF8)
+    wchar_t wTempDir[_MAX_PATH];
+    wchar_t wTempFileName[_MAX_PATH];
+    wchar_t wPrefix[_MAX_PATH];
+#endif
 
     memset(fname, 0, gp_file_name_sizeof);
     if (!gp_file_name_is_absolute(prefix, strlen(prefix))) {
-        int plen = sizeof(sTempDir);
+        int plen = _MAX_PATH;
 
-        if (gp_gettmpdir(sTempDir, &plen) != 0)
-            l = GetTempPath(sizeof(sTempDir), sTempDir);
-        else
+        /* gp_gettmpdir will always return a UTF8 encoded string unless
+         * GS_NO_UTF8 is defined, due to the windows specific version of
+         * gp_getenv being called (in gp_wgetv.c, not gp_getnv.c) */
+        if (gp_gettmpdir(sTempDir, &plen) != 0) {
+#ifdef METRO
+            /* METRO always uses UTF8 for 'ascii' - there is no concept of
+             * local encoding, so GS_NO_UTF8 makes no difference here. */
+            l = GetTempPathWRT(_MAX_PATH, wTempDir);
+            l = wchar_to_utf8(sTempDir, wTempDir);
+#elif defined(GS_NO_UTF8)
+            l = GetTempPathA(_MAX_PATH, sTempDir);
+#else
+            GetTempPathW(_MAX_PATH, wTempDir);
+            l = wchar_to_utf8(sTempDir, wTempDir);
+#endif
+        } else
             l = strlen(sTempDir);
     } else {
-        strncpy(sTempDir, prefix, sizeof(sTempDir));
+        strncpy(sTempDir, prefix, _MAX_PATH);
         prefix = "";
         l = strlen(sTempDir);
     }
@@ -612,8 +661,20 @@ gp_open_scratch_file(const gs_memory_t *mem,
     if (sTempDir[l-1] == '/')
         sTempDir[l-1] = '\\';		/* What Windoze prefers */
 
-    if (l <= sizeof(sTempDir)) {
-        n = GetTempFileName(sTempDir, prefix, 0, sTempFileName);
+    if (l <= _MAX_PATH) {
+#ifdef METRO
+        utf8_to_wchar(wTempDir, sTempDir);
+        utf8_to_wchar(wPrefix, prefix);
+        n = GetTempFileNameWRT(wTempDir, wPrefix, wTempFileName);
+        n = wchar_to_utf8(sTempFileName, wTempFileName);
+#elif defined(GS_NO_UTF8)
+        n = GetTempFileNameA(sTempDir, prefix, 0, sTempFileName);
+#else
+        utf8_to_wchar(wTempDir, sTempDir);
+        utf8_to_wchar(wPrefix, prefix);
+        GetTempFileNameW(wTempDir, wPrefix, 0, wTempFileName);
+        n = wchar_to_utf8(sTempFileName, wTempFileName);
+#endif
         if (n == 0) {
             /* If 'prefix' is not a directory, it is a path prefix. */
             int l = strlen(sTempDir), i;
@@ -627,11 +688,22 @@ gp_open_scratch_file(const gs_memory_t *mem,
                     break;
                 }
             }
-            if (i > 0)
-                n = GetTempFileName(sTempDir, sTempDir + i, 0, sTempFileName);
+            if (i > 0) {
+#ifdef METRO
+                utf8_to_wchar(wPrefix, sTempDir + i);
+                GetTempFileNameWRT(wTempDir, wPrefix, wTempFileName);
+                n = wchar_to_utf8(sTempFileName, wTempFileName);
+#elif defined(GS_NO_UTF8)
+                n = GetTempFileNameA(sTempDir, sTempDir + i, 0, sTempFileName);
+#else
+                utf8_to_wchar(wPrefix, sTempDir + i);
+                GetTempFileNameW(wTempDir, wPrefix, 0, wTempFileName);
+                n = wchar_to_utf8(sTempFileName, wTempFileName);
+#endif
+            }
         }
         if (n != 0) {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
             hfile = CreateFile(sTempFileName,
                                GENERIC_READ | GENERIC_WRITE | DELETE,
                                FILE_SHARE_READ | FILE_SHARE_WRITE,
@@ -645,12 +717,20 @@ gp_open_scratch_file(const gs_memory_t *mem,
                 hfile = INVALID_HANDLE_VALUE;
             else {
                 utf8_to_wchar(uni, sTempFileName);
+#ifdef METRO
+                hfile = CreateFile2(uni,
+                                    GENERIC_READ | GENERIC_WRITE | DELETE,
+                                    FILE_SHARE_READ | FILE_SHARE_WRITE,
+                                    CREATE_ALWAYS,
+                                    NULL);
+#else
                 hfile = CreateFileW(uni,
                                     GENERIC_READ | GENERIC_WRITE | DELETE,
                                     FILE_SHARE_READ | FILE_SHARE_WRITE,
                                     NULL, CREATE_ALWAYS,
                                     FILE_ATTRIBUTE_NORMAL /* | FILE_FLAG_DELETE_ON_CLOSE */,
                                     NULL);
+#endif
                 free(uni);
             }
 #endif
@@ -693,7 +773,7 @@ gp_open_scratch_file(const gs_memory_t *mem,
 FILE *
 gp_fopen(const char *fname, const char *mode)
 {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     return fopen(fname, mode);
 #else
     int len = utf8_to_wchar(NULL, fname);
@@ -760,9 +840,13 @@ FILE *gp_open_printer_64(const gs_memory_t *mem,
                                char         fname[gp_file_name_sizeof],
                                int          binary_mode)
 {
+#ifdef METRO
+    return NULL;
+#else
     /* Assuming gp_open_scratch_file_64 is same as gp_open_scratch_file -
        see the body of gp_open_printer. */
     return gp_open_printer(mem, fname, binary_mode);
+#endif
 }
 
 #if defined(_MSC_VER) && _MSC_VER < 1400
@@ -770,7 +854,7 @@ FILE *gp_open_printer_64(const gs_memory_t *mem,
     int _fseeki64( FILE *, int64_t, int);
 #endif
 
-int64_t gp_ftell_64(FILE *strm)
+gs_offset_t gp_ftell_64(FILE *strm)
 {
 #if !defined(_MSC_VER)
     return ftell(strm);
@@ -781,7 +865,7 @@ int64_t gp_ftell_64(FILE *strm)
 #endif
 }
 
-int gp_fseek_64(FILE *strm, int64_t offset, int origin)
+int gp_fseek_64(FILE *strm, gs_offset_t offset, int origin)
 {
 #if !defined(_MSC_VER)
     return fseek(strm, offset, origin);
@@ -796,6 +880,21 @@ int gp_fseek_64(FILE *strm, int64_t offset, int origin)
 #endif
 }
 
+bool gp_fseekable (FILE *f)
+{
+    struct stat s;
+    int fno;
+    
+    fno = fileno(f);
+    if (fno < 0)
+        return(false);
+    
+    if (fstat(fno, &s) < 0)
+        return(false);
+
+    return((bool)S_ISREG(s.st_mode));
+}
+
 /* -------------------------  _snprintf -----------------------------*/
 
 /* Microsoft Visual C++ 2005  doesn't properly define snprintf,
