@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Miscellaneous support for platform facilities */
 
 #include "unistd_.h"
@@ -246,7 +248,7 @@ gp_file_name_combine_generic(const char *prefix, uint plen, const char *fname, u
                 case 0:
                     /* We have no infix, start with parent. */
                     if ((no_sibling && ipe == fname + flen && flen != 0) ||
-                            !gp_file_name_is_partent_allowed())
+                            !gp_file_name_is_parent_allowed())
                         return gp_combine_cant_handle;
                     /* Falls through. */
                 case 2:
