@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Routines for determining equivalent color for spot colors */
 
 #include "math_.h"
@@ -26,6 +28,7 @@
 #include "gscspace.h"
 #include "gxcspace.h"
 #include "gsicc_manage.h"
+#include "gxdevsop.h"
 
 /*
  * These routines are part of the logic for determining an equivalent
@@ -147,6 +150,56 @@ update_Separation_spot_equivalent_cmyk_colors(gx_device * pdev,
     }
 }
 
+/* This is used for getting the equivalent CMYK values for the spots that
+   may exist in a DeviceN output ICC profile */
+static void
+update_ICC_spot_equivalent_cmyk_colors(gx_device * pdev,
+                    const gs_state * pgs, const gs_color_space * pcs,
+                    gs_devn_params * pdevn_params,
+                    equivalent_cmyk_color_params * pparams)
+{
+    int i, j;
+    cmm_dev_profile_t *dev_profile;
+    int code;
+    gs_client_color client_color;
+
+
+    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    /*
+     * Check if the ICC spot names matche any of the
+     * separations for which we need an equivalent CMYK color.
+     */
+    for (i = 0; i < pdevn_params->separations.num_separations; i++) {
+        if (pparams->color[i].color_info_valid == false) {
+            const devn_separation_name * dev_sep_name =
+                            &(pdevn_params->separations.names[i]);
+            gsicc_colorname_t *name_entry;
+
+            name_entry = dev_profile->spotnames->head;
+
+            for (j = 0; j < dev_profile->device_profile[0]->num_comps; j++) {
+                client_color.paint.values[j] = 0.0;
+            }
+            for (j = 0; j < dev_profile->spotnames->count; j++) {
+                if (compare_color_names(dev_sep_name->data, dev_sep_name->size,
+                                name_entry->name, name_entry->length)) {
+                    /*
+                     * Create a copy of the color space and then modify it
+                     * to force the use of the alternate color space.
+                     */
+                    client_color.paint.values[j] = 1.0;
+                    capture_spot_equivalent_cmyk_colors(pdev, pgs, &client_color,
+                                                pcs, i, pparams);
+                    break;
+                }
+                /* Did not match, grab the next one */
+                name_entry = name_entry->next;
+            }
+        }
+    }
+}
+
+
 static void
 update_DeviceN_spot_equivalent_cmyk_colors(gx_device * pdev,
                     const gs_state * pgs, const gs_color_space * pcs,
@@ -171,8 +224,18 @@ update_DeviceN_spot_equivalent_cmyk_colors(gx_device * pdev,
         pcs->params.device_n.get_colorname_string
             (pdev->memory, pcs->params.device_n.names[j],
              &pcs_sep_name, &cs_sep_name_size);
-        if (compare_color_names("None", 4, pcs_sep_name, cs_sep_name_size))
+        if (compare_color_names("None", 4, pcs_sep_name, cs_sep_name_size)) {
+            /* If we are going out to a device that supports devn colors
+               then it is possible that any preview that such a device creates
+               will be in error due to the lack of the proper mappings from
+               the full DeviceN colors (whicn can include one or more \None
+               enrties). Display a warning about this if in debug mode */
+#ifdef DEBUG
+            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_devn, NULL, 0))
+                gs_warn("Separation preview may be inaccurate due to presence of \\None colorants");
+#endif
             return;
+        }
     }
 
     /*
@@ -224,7 +287,10 @@ update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     gs_devn_params * pdevn_params, equivalent_cmyk_color_params * pparams)
 {
     const gs_color_space * pcs;
+    cmm_dev_profile_t *dev_profile;
+    int code;
 
+    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
     /* If all of the color_info is valid then there is nothing to do. */
     if (pparams->all_color_info_valid)
         return;
@@ -252,6 +318,14 @@ update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
                                                 pdevn_params, pparams);
             pparams->all_color_info_valid = check_all_colors_known
                     (pdevn_params->separations.num_separations, pparams);
+        } else if (pcs->type->index == gs_color_space_index_ICC &&
+            dev_profile->spotnames != NULL) {
+            /* In this case, we are trying to set up the equivalent colors
+               for the spots in the output ICC profile */
+            update_ICC_spot_equivalent_cmyk_colors(pdev, pgs, pcs, pdevn_params,
+                                                   pparams);
+            pparams->all_color_info_valid = check_all_colors_known
+                    (pdevn_params->separations.num_separations, pparams);
         }
     }
 }
@@ -329,7 +403,8 @@ cmap_rgb_capture_cmyk_color(frac r, frac g, frac b, gx_device_color * pdc,
 
 static void
 cmap_cmyk_capture_cmyk_color(frac c, frac m, frac y, frac k, gx_device_color * pdc,
-     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select,
+     const gs_color_space *pcs)
 {
     equivalent_cmyk_color_params * pparams =
             ((color_capture_device *)dev)->pequiv_cmyk_colors;
@@ -355,14 +430,14 @@ static void
 cmap_separation_capture_cmyk_color(frac all, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
-    dprintf("cmap_separation_capture_cmyk_color - this routine should not be executed\n");
+    dmprintf(pis->memory, "cmap_separation_capture_cmyk_color - this routine should not be executed\n");
 }
 
 static void
 cmap_devicen_capture_cmyk_color(const frac * pcc, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
-    dprintf("cmap_devicen_capture_cmyk_color - this routine should not be executed\n");
+    dmprintf(pis->memory, "cmap_devicen_capture_cmyk_color - this routine should not be executed\n");
 }
 
 /*
@@ -377,16 +452,24 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     gs_imager_state temp_state = *((const gs_imager_state *)pgs);
     color_capture_device temp_device = { 0 };
     gx_device_color dev_color;
-    gsicc_rendering_intents_t rendering_intent;
+    gsicc_rendering_param_t render_cond;
     int code;
     cmm_dev_profile_t *dev_profile;
     cmm_profile_t *curr_output_profile;
-    cmm_dev_profile_t temp_profile;
+    cmm_dev_profile_t temp_profile = {	/* Initialize to 0's/NULL's */
+                          { 0 } /* device_profile[] */, 0 /* proof_profile */,
+                          0 /* link_profile */, 0 /* oi_profile */, 
+                          { {0} } /* rendercond[] */, 0 /* devicegraytok */, 
+                          0 /* graydection */, 0 /* pageneutralcolor */, 
+                          0 /* usefastcolor */, 0 /* supports_devn */,
+                          0 /* sim_overprint */, 0 /* spotnames */, 
+                          0 /* prebandthreshold */, 0 /* memory */, 
+                          { 0 } /* rc_header */
+                          };
 
     code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
     gsicc_extract_profile(pdev->graphics_type_tag,
-                          dev_profile, &(curr_output_profile),
-                          &rendering_intent);
+                          dev_profile, &(curr_output_profile), &render_cond);
     /*
      * Create a temp device.  The primary purpose of this device is pass the
      * separation number and a pointer to the original device's equivalent
@@ -398,11 +481,30 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     temp_device.pequiv_cmyk_colors = pparams;
     temp_device.memory = pgs->memory;
 
-    temp_profile.device_profile[0] = curr_output_profile;
-    temp_profile.device_profile[1] = NULL;
-    temp_profile.device_profile[2] = NULL;
-    temp_profile.device_profile[2] = NULL;
+    temp_profile.usefastcolor = false;  /* This avoids a few headaches */
+    temp_profile.prebandthreshold = true;
+    temp_profile.supports_devn = false;
+    temp_profile.rendercond[0] = render_cond;
+    temp_profile.rendercond[1] = render_cond;
+    temp_profile.rendercond[2] = render_cond;
+    temp_profile.rendercond[3] = render_cond;
     temp_device.icc_struct = &temp_profile;
+
+    /* The equivalent CMYK colors are used for
+       the preview of the composite image and as such should not rely
+       upon the output profile if our output profile is a DeviceN profile.
+       For example, if our output profile was CMYK + OG then if we use
+       the device profile here we may not get CMYK values that
+       even come close to the color we want to simulate.  So, for this
+       case we go ahead and use the default CMYK profile and let the user
+       beware.  Note that the actual separations will be correct however
+       for the CMYK + OG values. */
+
+    if (curr_output_profile->data_cs == gsNCHANNEL) {
+        temp_profile.device_profile[0] = temp_state.icc_manager->default_cmyk;
+    } else {
+        temp_profile.device_profile[0] = curr_output_profile;
+    }
     set_dev_proc(&temp_device, get_profile, gx_default_get_profile);
 
     /*
@@ -417,5 +519,5 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
 
     /* Now capture the color */
     pcs->type->remap_color (pcc, pcs, &dev_color, &temp_state,
-                    (gx_device *)&temp_device, gs_color_select_texture);
+                            (gx_device *)&temp_device, gs_color_select_texture);
 }
