@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Implementation of LL3 Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -352,8 +354,6 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
         float vv0, vv1;
         double vb0, vb1;
 
-        if (v0 >= b1)
-            continue;
         if (v0 >= b1 - bsmall)
             continue; /* Ignore a small noise */
         vv0 = max(b0, v0);
@@ -364,13 +364,15 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
             return 1;
         if (vv0 < b1 && vv1 > b1) {
             *mask = 1;
-            return 0; /* Consider stitches as monotonity breaks. */
+            return 0; /* Consider stitches as monotony breaks. */
         }
         e0 = pfn->params.Encode[2 * i];
         e1 = pfn->params.Encode[2 * i + 1];
         esmall = (float)1e-6 * any_abs(e1 - e0);
         vb0 = max(vv0, b0);
         vb1 = min(vv1, b1);
+        if (b1 == b0)
+            return 1; /* function is monotonous in a point */
         w0 = (float)(vb0 - b0) * (e1 - e0) / (b1 - b0) + e0;
         w1 = (float)(vb1 - b0) * (e1 - e0) / (b1 - b0) + e0;
         /* Note that w0 > w1 is now possible if e0 > e1. */
