@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Adobe Type 1 charstring interpreter */
 #include "math_.h"
 #include "memory_.h"
@@ -204,8 +206,8 @@ gs_type1_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
                 }
             } else		/* not possible */
                 return_error(gs_error_invalidfont);
-          pushed:if_debug3('1', "[1]%d: (%d) %f\n",
-                      (int)(csp - cstack), c, fixed2float(*csp));
+          pushed:if_debug3m('1', pfont->memory, "[1]%d: (%d) %f\n",
+                            (int)(csp - cstack), c, fixed2float(*csp));
             continue;
         }
 #ifdef DEBUG
@@ -214,9 +216,9 @@ gs_type1_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
             {char1_command_names};
 
             if (c1names[c] == 0)
-                dlprintf2("[1]0x%lx: %02x??\n", (ulong) (cip - 1), c);
+                dmlprintf2(pfont->memory, "[1]0x%lx: %02x??\n", (ulong) (cip - 1), c);
             else
-                dlprintf3("[1]0x%lx: %02x %s\n", (ulong) (cip - 1), c,
+                dmlprintf3(pfont->memory, "[1]0x%lx: %02x %s\n", (ulong) (cip - 1), c,
                           c1names[c]);
         }
 #endif
@@ -386,10 +388,11 @@ rsbw:		/* Give the caller the opportunity to intervene. */
                     {char1_extended_command_names};
 
                     if (ce1names[c] == 0)
-                        dlprintf2("[1]0x%lx: %02x??\n", (ulong) (cip - 1), c);
+                        dmlprintf2(pfont->memory, "[1]0x%lx: %02x??\n",
+                                   (ulong) (cip - 1), c);
                     else
-                        dlprintf3("[1]0x%lx: %02x %s\n", (ulong) (cip - 1), c,
-                                  ce1names[c]);
+                        dmlprintf3(pfont->memory, "[1]0x%lx: %02x %s\n",
+                                   (ulong) (cip - 1), c, ce1names[c]);
                 }
 #endif
                 switch ((char1_extended_command) c) {
