@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Default implementation of text writing */
 #include "gx.h"
 #include "memory_.h"
@@ -33,10 +35,6 @@
 #include "gzpath.h"
 #include "gxfcid.h"
 
-/* Define the maximum size of a full temporary bitmap when rasterizing, */
-/* in bits (not bytes). */
-static const uint MAX_TEMP_BITMAP_BITS = 80000;
-
 /* Define whether the show operation uses the character outline data, */
 /* as opposed to just needing the width (or nothing). */
 #define SHOW_USES_OUTLINE(penum)\
@@ -279,7 +277,7 @@ gx_default_text_restore_state(gs_text_enum_t *pte)
 
 static int
     set_cache_device(gs_show_enum *penum, gs_state *pgs,
-                     floatp llx, floatp lly, floatp urx, floatp ury);
+                     double llx, double lly, double urx, double ury);
 
 /* This is the default implementation of text enumerator set_cache. */
 static int
@@ -364,7 +362,7 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 /* Note that this returns 1 if the current show operation is */
 /* non-displaying (stringwidth or cshow). */
 int
-set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
+set_char_width(gs_show_enum *penum, gs_state *pgs, double wx, double wy)
 {
     int code;
 
@@ -487,8 +485,8 @@ compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *al
 /* Return 1 if we just set up a cache device. */
 /* Used by setcachedevice and setcachedevice2. */
 static int
-set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
-                 floatp urx, floatp ury)
+set_cache_device(gs_show_enum * penum, gs_state * pgs, double llx, double lly,
+                 double urx, double ury)
 {
     gs_glyph glyph;
 
@@ -497,7 +495,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         return 0;
     if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE)) { /* cshow */
         int code;
-        if_debug0('k', "[k]no cache: cshow");
+        if_debug0m('k', penum->memory, "[k]no cache: cshow");
         code = gs_nulldevice(pgs);
         if (code < 0)
             return code;
@@ -511,8 +509,8 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
     /* We can only use the cache if ctm is unchanged */
     /* (aside from a possible translation). */
     if (penum->can_cache <= 0 || !pgs->char_tm_valid) {
-        if_debug2('k', "[k]no cache: can_cache=%d, char_tm_valid=%d\n",
-                  penum->can_cache, (int)pgs->char_tm_valid);
+        if_debug2m('k', penum->memory, "[k]no cache: can_cache=%d, char_tm_valid=%d\n",
+                   penum->can_cache, (int)pgs->char_tm_valid);
         return 0;
     } {
         const gs_font *pfont = pgs->font;
@@ -531,13 +529,19 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         cached_char *cc;
         gs_fixed_rect clip_box;
         int code;
+        gs_fixed_point cll, clr, cul, cur, cdim;
+
+        /* Reject setcachedevice arguments that are too big and, probably, invalid */
+        /* The threshold is arbitrary. A font from bug 692832 has a 1237340,       */
+        /* normal fonts should have about 1000. */
+        if (fabs(llx) > 32000. || fabs(lly) > 32000. || fabs(urx) > 32000. || fabs(ury) >= 32000.)
+            return 0;           /* don't cache */
 
         /* Compute the bounding box of the transformed character. */
         /* Since we accept arbitrary transformations, the extrema */
         /* may occur in any order; however, we can save some work */
         /* by observing that opposite corners before transforming */
         /* are still opposite afterwards. */
-        gs_fixed_point cll, clr, cul, cur, cdim;
 
         if ((code = gs_distance_transform2fixed(&pgs->ctm, llx, lly, &cll)) < 0 ||
             (code = gs_distance_transform2fixed(&pgs->ctm, llx, ury, &clr)) < 0 ||
@@ -570,13 +574,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
             return code;
 #ifdef DEBUG
         if (gs_debug_c('k')) {
-            dlprintf6("[k]cbox=[%g %g %g %g] scale=%dx%d\n",
-                      fixed2float(cll.x), fixed2float(cll.y),
-                      fixed2float(cur.x), fixed2float(cur.y),
-                      1 << log2_scale.x, 1 << log2_scale.y);
-            dlprintf6("[p]  ctm=[%g %g %g %g %g %g]\n",
-                      pgs->ctm.xx, pgs->ctm.xy, pgs->ctm.yx, pgs->ctm.yy,
-                      pgs->ctm.tx, pgs->ctm.ty);
+            dmlprintf6(pgs->memory, "[k]cbox=[%g %g %g %g] scale=%dx%d\n",
+                       fixed2float(cll.x), fixed2float(cll.y),
+                       fixed2float(cur.x), fixed2float(cur.y),
+                       1 << log2_scale.x, 1 << log2_scale.y);
+            dmlprintf6(pgs->memory, "[p]  ctm=[%g %g %g %g %g %g]\n",
+                       pgs->ctm.xx, pgs->ctm.xy, pgs->ctm.yx, pgs->ctm.yy,
+                       pgs->ctm.tx, pgs->ctm.ty);
         }
 #endif
         cdim.x = cur.x - cll.x;
@@ -587,9 +591,9 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
             return 0;           /* much too big */
         iwidth = ((ushort) fixed2int_var(cdim.x) + 3) << log2_scale.x;
         iheight = ((ushort) fixed2int_var(cdim.y) + 3) << log2_scale.y;
-        if_debug3('k', "[k]iwidth=%u iheight=%u dev_cache %s\n",
-                  (uint) iwidth, (uint) iheight,
-                  (penum->dev_cache == 0 ? "not set" : "set"));
+        if_debug3m('k', penum->memory, "[k]iwidth=%u iheight=%u dev_cache %s\n",
+                   (uint) iwidth, (uint) iheight,
+                   (penum->dev_cache == 0 ? "not set" : "set"));
         if (penum->dev_cache == 0) {
             code = show_cache_setup(penum);
             if (code < 0)
@@ -603,7 +607,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
          * full oversampling with compression at the end.
          */
         code = gx_alloc_char_bits(dir, penum->dev_cache,
-                                (iwidth > MAX_TEMP_BITMAP_BITS / iheight &&
+                                (iwidth > MAX_CCACHE_TEMP_BITMAP_BITS / iheight &&
                                  log2_scale.x + log2_scale.y > alpha_bits ?
                                  penum->dev_cache2 : NULL),
                                 iwidth, iheight, &log2_scale, depth, &cc);
@@ -640,10 +644,10 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         /* Truncate the offsets to avoid artifacts later. */
         cc->offset.x = fixed_ceiling(-cll.x) + fixed_1;
         cc->offset.y = fixed_ceiling(-cll.y) + fixed_1;
-        if_debug4('k', "[k]width=%u, height=%u, offset=[%g %g]\n",
-                  (uint) iwidth, (uint) iheight,
-                  fixed2float(cc->offset.x),
-                  fixed2float(cc->offset.y));
+        if_debug4m('k', penum->memory, "[k]width=%u, height=%u, offset=[%g %g]\n",
+                   (uint) iwidth, (uint) iheight,
+                   fixed2float(cc->offset.x),
+                   fixed2float(cc->offset.y));
         pgs->in_cachedevice = CACHE_DEVICE_NONE; /* Provide correct grestore */
         if ((code = gs_gsave(pgs)) < 0) {
             gx_free_cached_char(dir, cc);
@@ -681,9 +685,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         clip_box.q.y = int2fixed(iheight);
         if ((code = gx_clip_to_rectangle(pgs, &clip_box)) < 0)
             return code;
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
         pgs->in_cachedevice = CACHE_DEVICE_CACHING;
     }
@@ -907,8 +915,13 @@ show_move(gs_show_enum * penum)
             return code;
     } else {
         double dx = 0, dy = 0;
-
-        if (SHOW_IS_ADD_TO_SPACE(penum)) {
+        /* Specifically for applying PDF word spacing, if single_byte_space == true
+           we'll only apply the delta for single byte character codes == space.s_char
+           See psi/zchar.c zpdfwidthshow and zpdfawidthshow for more detail
+         */
+        if (SHOW_IS_ADD_TO_SPACE(penum)
+            && (!penum->single_byte_space
+            || penum->bytes_decoded == 1)) {
             gs_char chr = gx_current_char((const gs_text_enum_t *)penum);
 
             if (chr == penum->text.space.s_char) {
@@ -971,7 +984,7 @@ show_proceed(gs_show_enum * penum)
   (++(penum->xy_index), next_char_glyph(pte, pchr, pglyph))
     gs_char chr;
     gs_glyph glyph;
-    int code;
+    int code, start;
     cached_char *cc;
     gs_log2_scale_point log2_scale;
 
@@ -989,6 +1002,7 @@ show_proceed(gs_show_enum * penum)
     if (penum->can_cache >= 0) {
         /* Loop with cache */
         for (;;) {
+            start = penum->index;
             switch ((code = get_next_char_glyph((gs_text_enum_t *)penum,
                                                 &chr, &glyph))
                     ) {
@@ -1022,6 +1036,7 @@ show_proceed(gs_show_enum * penum)
                         SET_CURRENT_GLYPH(penum, glyph);
                     } else
                         SET_CURRENT_GLYPH(penum, glyph);
+                    penum->bytes_decoded = penum->index - start;
                     penum->is_pure_color = gs_color_writes_pure(penum->pgs); /* Save
                                  this data for compute_glyph_raster_params to work
                                  independently on the color change in BuildChar.
