@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Shared procedures for outline character rendering */
 
 #ifndef gxchrout_INCLUDED
@@ -28,6 +30,6 @@ typedef struct gs_imager_state_s gs_imager_state;
  * The second argument is the default scaling for the font: 0.001 for
  * Type 1 fonts, 1.0 for TrueType fonts.
  */
-double gs_char_flatness(const gs_imager_state *pis, floatp default_scale);
+double gs_char_flatness(const gs_imager_state *pis, double default_scale);
 
 #endif /* gxchrout_INCLUDED */
