@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Internal definitions for Ghostscript command lists. */
 
 #ifndef gxcldev_INCLUDED
@@ -35,8 +37,9 @@
 /*#define cmd_compress_none 0 *//* (implicit) */
 #define cmd_compress_rle 1
 #define cmd_compress_cfe 2
+#define cmd_compress_const 3
 #define cmd_mask_compress_any\
-  ((1 << cmd_compress_rle) | (1 << cmd_compress_cfe))
+  ((1 << cmd_compress_rle) | (1 << cmd_compress_cfe) | (1 << cmd_compress_const))
 /* Exported by gxclutil.c */
 void clist_rle_init(stream_RLE_state *ss);
 void clist_rld_init(stream_RLD_state *ss);
@@ -44,6 +47,15 @@ void clist_cfe_init(stream_CFE_state *ss, int width, gs_memory_t *mem);
 void clist_cfd_init(stream_CFD_state *ss, int width, int height,
                     gs_memory_t *mem);
 
+/* Write out a block of data of arbitrary size and content to a      */
+/* specified band at the given offset PAST the last band in the page */
+/* Used for various data such as the icc profile table and the       */
+/* per band color_usage array                                        */
+/*                                                                   */
+/* If the size is not well defined, then the data itself should      */
+/* contain information for length or logical end-of-data.            */
+int cmd_write_pseudo_band(gx_device_clist_writer *cldev, unsigned char *pbuf,
+                          int data_size, int pseudo_band_offset);
 /*
  * A command always consists of an operation followed by operands;
  * the syntax of the operands depends on the operation.
@@ -133,8 +145,15 @@ extern const char *const *const cmd_sub_op_names[16];
  */
 #define cmd_max_intsize(siz)\
   (((siz) * 8 + 6) / 7)
+
+/* NB: Assume that the largest size is for dash patterns. Larger that this
+ *     need to be read from the cbuf in a loop.
+ */
 #define cmd_largest_size\
-  (2 + (1 + cmd_max_dash) * sizeof(float))
+  (2 + sizeof(float)		/* dot_length */\
+     + sizeof(float)           /* offset */\
+     + (cmd_max_dash * sizeof(float))\
+  )
 
 /* ---------------- Command parameters ---------------- */
 
@@ -226,8 +245,6 @@ typedef struct cmd_block_s {
     int band_min, band_max;
 #define cmd_band_end (-1)	/* end of band file */
     int64_t pos;		/* starting position in cfile */
-    gx_band_complexity_t band_complexity;
-
 } cmd_block;
 
 /* ---------------- Band state ---------------- */
@@ -248,6 +265,7 @@ struct gx_clist_state_s {
     gs_id pattern_id;		/* the last stored pattern id. */
     gs_int_point tile_phase;	/* most recent tile phase */
     gx_color_index tile_colors[2];	/* most recent tile colors */
+    gx_device_color tile_color_devn[2];  /* devn tile colors */
     gx_cmd_rect rect;		/* most recent rectangle */
     gs_logical_operation_t lop;	/* most recent logical op */
     short lop_enabled;		/* 0 = don't use lop, 1 = use lop, */
@@ -255,6 +273,7 @@ struct gx_clist_state_s {
     short clip_enabled;		/* 0 = don't clip, 1 = do clip, */
                                 /* -1 is used internally */
     bool color_is_alpha;	/* for copy_color_alpha */
+    bool color_is_devn;     /* more overload of copy_color_alpha for devn support */ 
     uint known;			/* flags for whether this band */
                                 /* knows various misc. parameters */
     /* We assign 'known' flags here from the high end; */
@@ -265,8 +284,7 @@ struct gx_clist_state_s {
     /* Following are only used when writing */
     cmd_list list;		/* list of commands for band */
     /* Following are set when writing, read when reading */
-    gx_band_complexity_t band_complexity;
-    gx_colors_used_t colors_used;
+    gx_color_usage_t color_usage;
 };
 
 /* The initial values for a band state */
@@ -276,13 +294,19 @@ struct gx_clist_state_s {
         { gx_dc_type_none },\
         0, gx_no_bitmap_id, gs_no_id,\
          { 0, 0 }, { gx_no_color_index, gx_no_color_index },\
-         { 0, 0, 0, 0 }, lop_default, 0, 0, 0, initial_known,\
-        { 0, 0 }, { 0, 0 }, { 0, 0 }
+        { {NULL}, {NULL} },\
+         { 0, 0, 0, 0 }, lop_default, 0, 0, 0, 0, initial_known,\
+        { 0, 0 }, /* cmd_list */\
+        { 0, /* or */\
+          0, /* slow rop */\
+          { { max_int, max_int }, /* p */ { min_int, min_int } /* q */ } /* trans_bbox */\
+        } /* color_usage */
 
 /* Define the size of the command buffer used for reading. */
+#define cbuf_size 4096
 /* This is needed to split up operations with a large amount of data, */
 /* primarily large copy_ operations. */
-#define cbuf_size 4096
+#define data_bits_size 4096
 
 /* ---------------- Driver procedures ---------------- */
 
@@ -293,6 +317,7 @@ dev_proc_copy_mono(clist_copy_mono);
 dev_proc_copy_color(clist_copy_color);
 dev_proc_copy_alpha(clist_copy_alpha);
 dev_proc_strip_tile_rectangle(clist_strip_tile_rectangle);
+dev_proc_strip_tile_rect_devn(clist_strip_tile_rect_devn);
 dev_proc_strip_copy_rop(clist_strip_copy_rop);
 dev_proc_strip_copy_rop2(clist_strip_copy_rop2);
 dev_proc_fill_trapezoid(clist_fill_trapezoid);
@@ -300,6 +325,9 @@ dev_proc_fill_linear_color_trapezoid(clist_fill_linear_color_trapezoid);
 dev_proc_fill_linear_color_triangle(clist_fill_linear_color_triangle);
 dev_proc_dev_spec_op(clist_dev_spec_op);
 dev_proc_copy_planes(clist_copy_planes);
+dev_proc_fill_rectangle_hl_color(clist_fill_rectangle_hl_color);
+dev_proc_copy_alpha_hl_color(clist_copy_alpha_hl_color);
+dev_proc_process_page(clist_process_page);
 
 /* In gxclimag.c */
 dev_proc_fill_mask(clist_fill_mask);
@@ -377,13 +405,13 @@ int clist_VMerror_recover_flush(gx_device_clist_writer *, int);
 int cmd_put_params(gx_device_clist_writer *, gs_param_list *);
 
 /* Conditionally keep command statistics. */
-#ifdef DEBUG
-int cmd_count_op(int op, uint size);
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
+int cmd_count_op(int op, uint size, const gs_memory_t *mem);
 void cmd_uncount_op(int op, uint size);
-void cmd_print_stats(void);
+void cmd_print_stats(const gs_memory_t *);
 #  define cmd_count_add1(v) (v++)
 #else
-#  define cmd_count_op(op, size) (op)
+#  define cmd_count_op(op, size, mem) (op)
 #  define cmd_uncount_op(op, size) DO_NOTHING
 #  define cmd_count_add1(v) DO_NOTHING
 #endif
@@ -407,7 +435,7 @@ byte *cmd_put_op(gx_device_clist_writer * cldev, gx_clist_state * pcls, uint siz
 #define set_cmd_put_op(dp, cldev, pcls, op, csize)\
   ( (dp = cmd_put_op(cldev, pcls, csize)) == 0 ?\
       (cldev)->error_code :\
-    (*dp = cmd_count_op(op, csize), 0) )
+    (*dp = cmd_count_op(op, csize, cldev->memory), 0) )
 
 /* Add a command for all bands or a range of bands. */
 byte *cmd_put_range_op(gx_device_clist_writer * cldev, int band_min,
@@ -419,7 +447,7 @@ byte *cmd_put_range_op(gx_device_clist_writer * cldev, int band_min,
 #define set_cmd_put_range_op(dp, cldev, op, bmin, bmax, csize)\
   ( (dp = cmd_put_range_op(cldev, bmin, bmax, csize)) == 0 ?\
       (cldev)->error_code :\
-    (*dp = cmd_count_op(op, csize), 0) )
+    (*dp = cmd_count_op(op, csize, (cldev)->memory), 0) )
 #define set_cmd_put_all_op(dp, cldev, op, csize)\
   set_cmd_put_range_op(dp, cldev, op, 0, (cldev)->nbands - 1, csize)
 
@@ -473,10 +501,13 @@ typedef struct {
     byte set_op;
     byte delta_op;
     bool tile_color;
+    bool devn;
 } clist_select_color_t;
+
 extern const clist_select_color_t
       clist_select_color0, clist_select_color1, clist_select_tile_color0,
-      clist_select_tile_color1;
+      clist_select_tile_color1, clist_select_devn_color0, 
+      clist_select_devn_color1;
 
 /* See comments in gxclutil.c */
 int cmd_put_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
@@ -609,7 +640,7 @@ typedef struct cmd_rects_enum_s {
         int band;
         gx_clist_state *pcls;
         int band_end;
-        int nbands;
+        int rect_nbands;
 } cmd_rects_enum_t;
 
 #define RECT_ENUM_INIT(re, yvar, heightvar)\
@@ -617,7 +648,7 @@ typedef struct cmd_rects_enum_s {
         re.height = heightvar;\
         re.yend = re.y + re.height;\
         re.band_height = cdev->page_band_height;\
-        re.nbands = (re.yend - re.y + re.band_height - 1) / re.band_height;
+        re.rect_nbands = (re.yend - re.y + re.band_height - 1) / re.band_height;
 
 #define RECT_STEP_INIT(re)\
             re.band = re.y / re.band_height;\
@@ -636,7 +667,10 @@ const byte *cmd_read_matrix(gs_matrix * pmat, const byte * cbp);
 /* Put out a fill or tile rectangle command. */
 int cmd_write_rect_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                        int op, int x, int y, int width, int height);
-
+/* Put out a fill with a devn color */
+int cmd_write_rect_hl_cmd(gx_device_clist_writer * cldev, 
+                             gx_clist_state * pcls, int op, int x, int y, 
+                             int width, int height, bool extended_command);
 /* Put out a fill or tile rectangle command for fillpage. */
 int cmd_write_page_rect_cmd(gx_device_clist_writer * cldev, int op);
 
@@ -728,6 +762,7 @@ int cmd_put_halftone(gx_device_clist_writer * cldev,
  */
 typedef enum {
     playback_action_render,
+    playback_action_render_no_pdf14,
     playback_action_setup
 } clist_playback_action;
 
@@ -752,18 +787,20 @@ int clist_writer_push_cropping(gx_device_clist_writer *cdev, int ry, int rheight
 int clist_writer_pop_cropping(gx_device_clist_writer *cdev);
 int clist_writer_check_empty_cropping_stack(gx_device_clist_writer *cdev);
 int clist_read_icctable(gx_device_clist_reader *crdev);
+int clist_read_color_usage_array(gx_device_clist_reader *crdev);
 
 /* Special write out for the serialized icc profile table */
 
 int cmd_write_icctable(gx_device_clist_writer * cldev, unsigned char *pbuf, int data_size);
 
 /* Enumeration of psuedo band offsets for extra c-list data.
-   This includes the ICC profile table and may later include
-   compressed image data */
+   This includes the ICC profile table and and color_usage and
+   may later include compressed image data */
 
 typedef enum {
 
-    ICC_BAND_OFFSET = 1
+    COLOR_USAGE_OFFSET = 1,
+    ICC_TABLE_OFFSET = 2
 
 } psuedoband_offset;
 
