@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Higher-level path operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -26,6 +28,7 @@
 #include "gxclpath.h"
 #include "gxcolor2.h"
 #include "gxdcolor.h"
+#include "gxpcolor.h"
 #include "gxpaint.h"		/* for gx_fill/stroke_params */
 #include "gzpath.h"
 #include "gzcpath.h"
@@ -34,7 +37,7 @@
 #include "gxdevsop.h"
 
 /* Statistics */
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
 ulong stats_cmd_diffs[5];
 #endif
 
@@ -48,7 +51,7 @@ static int cmd_put_path(gx_device_clist_writer * cldev,
 
 /* Compute the colors used by a colored halftone. */
 static gx_color_index
-colored_halftone_colors_used(gx_device_clist_writer *cldev,
+colored_halftone_color_usage(gx_device_clist_writer *cldev,
                              const gx_drawing_color *pdcolor)
 {
     /*
@@ -100,7 +103,8 @@ cmd_slow_rop(gx_device *dev, gs_logical_operation_t lop,
 /* If the pattern color is big, it can write to "all" bands. */
 int
 cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
-                      const gx_drawing_color * pdcolor, cmd_rects_enum_t *pre)
+                      const gx_drawing_color * pdcolor, cmd_rects_enum_t *pre,
+                      dc_devn_cl_type devn_type)
 {
     const gx_device_halftone * pdht = pdcolor->type->get_dev_halftone(pdcolor);
     int                        code, di;
@@ -131,6 +135,7 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
      *
      * The complete cmd_opv_ext_put_drawing_color consists of:
      *  comand code (2 bytes)
+     *  tile index value or non tile color (1)
      *  device color type index (1)
      *  length of serialized device color (enc_u_sizew(dc_size))
      *  the serialized device color itself (dc_size)
@@ -144,11 +149,11 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                                  &dc_size );
 
     /* if the returned value is > 0, no change in the color is necessary */
-    if (code > 0)
+    if (code > 0 && devn_type == devn_not_tile)
         return 0;
     else if (code < 0 && code != gs_error_rangecheck)
         return code;
-    if (!all_bands && dc_size * pre->nbands > 1024*1024 /* arbitrary */)
+    if (!all_bands && dc_size * pre->rect_nbands > 1024*1024 /* arbitrary */)
         all_bands = true;
     is_pattern = gx_dc_is_pattern1_color(pdcolor);
     if (is_pattern)
@@ -206,7 +211,19 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         if (code < 0)
             return code;
         dp0 = dp;
-        dp[1] = cmd_opv_ext_put_drawing_color;
+        switch (devn_type) {
+            case devn_not_tile:
+                dp[1] = cmd_opv_ext_put_drawing_color;
+                break;
+            case devn_tile0:
+                dp[1] = cmd_opv_ext_put_tile_devn_color0;
+                break;
+            case devn_tile1:
+                dp[1] = cmd_opv_ext_put_tile_devn_color1;
+                break;
+            default:
+                dp[1] = cmd_opv_ext_put_drawing_color;
+        }
         dp += 2;
         *dp++ = di | (offset > 0 ? 0x80 : 0);
         if (offset > 0)
@@ -227,14 +244,8 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         left -= portion_size;
     } while (left);
 
-    /* should properly calculate colors_used, but for now just punt */
-    pcls->colors_used.or = ((gx_color_index)1 << cldev->clist_color_info.depth) - 1;
-
-    /* Here we can't know whether a pattern paints colors besides
-       black and white, so assume that it does.
-       todo: provide this info with a pattern tile. */
-    pcls->band_complexity.uses_color |= is_pattern ||
-                (pdcolor->colors.pure != 0 && pdcolor->colors.pure != 0xffffff);
+    /* attempt to properly calculate color_usage */
+    pcls->color_usage.or |= cmd_drawing_color_usage(cldev, pdcolor);
 
     /* record the color we have just serialized color */
     pdcolor->type->save_dc(pdcolor, &pcls->sdc);
@@ -246,6 +257,23 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         /* HACK: since gx_dc_pattern_write identifies pattern by tile id,
            replace the client's pattern id with tile id in the saved color.  */
         pcls->sdc.colors.pattern.id = pattern_id;
+        if (pattern_id &&
+            (gx_pattern1_get_transptr(pdcolor) != NULL ||
+             gx_pattern1_clist_has_trans(pdcolor))) {
+            /* update either this band or all bands with the trans_bbox */
+            if (all_bands) {
+                pcls->color_usage.trans_bbox.p.x = 0;
+                pcls->color_usage.trans_bbox.q.x = cldev->width;  /* no other information available */
+                pcls->color_usage.trans_bbox.p.y = 0;
+                pcls->color_usage.trans_bbox.q.y = cldev->height;
+                clist_update_trans_bbox(cldev, &(pcls->color_usage.trans_bbox));
+            } else {
+                pcls->color_usage.trans_bbox.p.x = 0;
+                pcls->color_usage.trans_bbox.q.x = cldev->width;  /* no other information available */
+                pcls->color_usage.trans_bbox.p.y = pre->y;
+                pcls->color_usage.trans_bbox.q.y = pre->yend;
+            }
+        }
     }
     if (is_pattern && all_bands) {
         /* Distribute the written pattern params to all bands.
@@ -264,18 +292,26 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 }
 
 /* Compute the colors used by a drawing color. */
-gx_color_index
-cmd_drawing_colors_used(gx_device_clist_writer *cldev,
+gx_color_usage_bits
+cmd_drawing_color_usage(gx_device_clist_writer *cldev,
                         const gx_drawing_color * pdcolor)
 {
     if (gx_dc_is_pure(pdcolor))
-        return gx_dc_pure_color(pdcolor);
+        return gx_color_index2usage((gx_device *)cldev, gx_dc_pure_color(pdcolor));
     else if (gx_dc_is_binary_halftone(pdcolor))
-        return gx_dc_binary_color0(pdcolor) | gx_dc_binary_color1(pdcolor);
+        return gx_color_index2usage((gx_device *)cldev,
+                                    gx_color_index2usage((gx_device *)cldev, gx_dc_binary_color0(pdcolor)) |
+                                    gx_color_index2usage((gx_device *)cldev, gx_dc_binary_color1(pdcolor)));
     else if (gx_dc_is_colored_halftone(pdcolor))
-        return colored_halftone_colors_used(cldev, pdcolor);
+        return gx_color_index2usage((gx_device *)cldev, colored_halftone_color_usage(cldev, pdcolor));
+    else if (gx_dc_is_devn(pdcolor)) {
+        gx_color_usage_bits bits = 0;
+
+        gx_dc_devn_get_nonzero_comps(pdcolor, (gx_device *)cldev, (int *)&bits);
+        return bits;
+    }
     else
-        return ((gx_color_index)1 << cldev->clist_color_info.depth) - 1;
+        return gx_color_usage_all(cldev);
 }
 
 /* Clear (a) specific 'known' flag(s) for all bands. */
@@ -315,7 +351,7 @@ cmd_check_clip_path(gx_device_clist_writer * cldev, const gx_clip_path * pcpath)
   shape_alpha_known | fill_adjust_known | alpha_known | clip_path_known)
 static void
 cmd_check_fill_known(gx_device_clist_writer *cdev, const gs_imager_state *pis,
-                     floatp flatness, const gs_fixed_point *padjust,
+                     double flatness, const gs_fixed_point *padjust,
                      const gx_clip_path *pcpath, uint *punknown)
 {
     /*
@@ -675,7 +711,7 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     gx_device_clist_writer * const cdev =
         &((gx_device_clist *)dev)->writer;
     uint unknown = 0;
-    int ry, rheight, y0, y1;
+    int ry, rheight, rx, rwidth, y0, y1;
     gs_logical_operation_t lop = pis->log_op;
     byte op = (byte)
         (params->rule == gx_rule_even_odd ?
@@ -700,6 +736,9 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
         crop_fill_y(cdev, ry, rheight);
         if (rheight <= 0)
             return 0;
+        rx = fixed2int(bbox.p.x) - 1;
+        rwidth = fixed2int_ceiling(bbox.q.x) - rx + 1;
+        fit_fill_w(cdev, rx, rwidth);
     }
     if ( (cdev->disable_mask & clist_disable_fill_path) ||
          gs_debug_c(',')
@@ -722,8 +761,9 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
         if (cdev->cropping_saved) {
             cdev->cropping_min = cdev->save_cropping_min;
             cdev->cropping_max = cdev->save_cropping_max;
-            if_debug2('v', "[v] clist_fill_path: restore cropping_min=%d croping_max=%d\n",
-                                    cdev->save_cropping_min, cdev->save_cropping_max);
+            if_debug2m('v', cdev->memory,
+                       "[v] clist_fill_path: restore cropping_min=%d croping_max=%d\n",
+                       cdev->save_cropping_min, cdev->save_cropping_max);
         }
         return code;
     }
@@ -746,8 +786,9 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
         cdev->save_cropping_max = cdev->cropping_max;
         cdev->cropping_min = max(ry, cdev->cropping_min);
         cdev->cropping_max = min(ry + rheight, cdev->cropping_max);
-        if_debug2('v', "[v] clist_fill_path: narrow cropping_min=%d croping_max=%d\n",
-                                cdev->save_cropping_min, cdev->save_cropping_max);
+        if_debug2m('v', cdev->memory,
+                   "[v] clist_fill_path: narrow cropping_min=%d croping_max=%d\n",
+                   cdev->save_cropping_min, cdev->save_cropping_max);
         RECT_ENUM_INIT(re, ry, rheight);
         do {
             RECT_STEP_INIT(re);
@@ -762,6 +803,18 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
             re.y += re.height;
         } while (re.y < re.yend);
     } else {
+        /* If needed, update the trans_bbox */
+        if (cdev->pdf14_needed) {
+            gs_int_rect bbox;
+
+            bbox.p.x = rx;
+            bbox.q.x = rx + rwidth - 1;
+            bbox.p.y = ry;
+            bbox.q.y = ry + rheight - 1;
+
+            clist_update_trans_bbox(cdev, &bbox);
+        }
+
         RECT_ENUM_INIT(re, ry, rheight);
         do {
             RECT_STEP_INIT(re);
@@ -772,7 +825,8 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
                 (code = cmd_update_lop(cdev, re.pcls, lop)) < 0
                 )
                 return code;
-            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re);
+            code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
+                                         devn_not_tile);
             if (code == gs_error_unregistered)
                 return code;
             if (code < 0) {
@@ -780,7 +834,7 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
                 return gx_default_fill_path(dev, pis, ppath, params, pdcolor,
                                             pcpath);
             }
-            re.pcls->colors_used.slow_rop |= slow_rop;
+            re.pcls->color_usage.slow_rop |= slow_rop;
             code = cmd_put_path(cdev, re.pcls, ppath,
                                 int2fixed(max(re.y - 1, y0)),
                                 int2fixed(min(re.y + re.height + 1, y1)),
@@ -905,6 +959,20 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
         cmd_clear_known(cdev, unknown);
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect trans_bbox;
+        int rx = fixed2int(bbox.p.x) - 1;
+        int rwidth = fixed2int_ceiling(bbox.q.x) - rx + 1;
+
+        fit_fill_w(cdev, rx, rwidth);
+        trans_bbox.p.x = rx;
+        trans_bbox.q.x = rx + rwidth - 1;
+        trans_bbox.p.y = ry;
+        trans_bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &trans_bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         int code;
@@ -915,7 +983,7 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
             (code = cmd_update_lop(cdev, re.pcls, lop)) < 0
             )
             return code;
-        code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re);
+        code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, devn_not_tile);
             if (code == gs_error_unregistered)
                 return code;
         if (code < 0) {
@@ -923,7 +991,7 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
             return gx_default_stroke_path(dev, pis, ppath, params, pdcolor,
                                           pcpath);
         }
-        re.pcls->colors_used.slow_rop |= slow_rop;
+        re.pcls->color_usage.slow_rop |= slow_rop;
         {
             fixed ymin, ymax;
 
@@ -992,13 +1060,27 @@ clist_put_polyfill(gx_device *dev, fixed px, fixed py,
     y1 = ry + rheight;
     if (cdev->permanent_error < 0)
       return (cdev->permanent_error);
+    /* If needed, update the trans_bbox */
+    if (cdev->pdf14_needed) {
+        gs_int_rect trans_bbox;
+        int rx = fixed2int(bbox.p.x) - 1;
+        int rwidth = fixed2int_ceiling(bbox.q.x) - rx + 1;
+
+        fit_fill_w(cdev, rx, rwidth);
+        trans_bbox.p.x = rx;
+        trans_bbox.q.x = rx + rwidth - 1;
+        trans_bbox.p.y = ry;
+        trans_bbox.q.y = ry + rheight - 1;
+
+        clist_update_trans_bbox(cdev, &trans_bbox);
+    }
     RECT_ENUM_INIT(re, ry, rheight);
     do {
         RECT_STEP_INIT(re);
         if ((code = cmd_update_lop(cdev, re.pcls, lop)) < 0 ||
-            (code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re)) < 0)
+            (code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, devn_not_tile)) < 0)
             goto out;
-        re.pcls->colors_used.slow_rop |= slow_rop;
+        re.pcls->color_usage.slow_rop |= slow_rop;
         code = cmd_put_path(cdev, re.pcls, &path,
                             int2fixed(max(re.y - 1, y0)),
                             int2fixed(min(re.y + re.height + 1, y1)),
@@ -1091,11 +1173,11 @@ cmd_put_segment(cmd_segment_writer * psw, byte op,
     if (gs_debug_c('L')) {
         int j;
 
-        dlprintf2("[L]  %s:%d:", cmd_sub_op_names[op >> 4][op & 0xf],
+        dmlprintf2(psw->cldev->memory, "[L]  %s:%d:", cmd_sub_op_names[op >> 4][op & 0xf],
                   (int)notes);
         for (j = 0; j < i; ++j)
-            dprintf1(" %g", fixed2float(operands[j]));
-        dputs("\n");
+            dmprintf1(psw->cldev->memory, " %g", fixed2float(operands[j]));
+        dmputs(psw->cldev->memory, "\n");
     }
 #endif
 
@@ -1127,7 +1209,7 @@ cmd_put_segment(cmd_segment_writer * psw, byte op,
                         operands[1] == -psw->delta_first.y
                         ) {
                         cmd_uncount_op(cmd_opv_rm2lineto, psw->len);
-                        *psw->dp = cmd_count_op(cmd_opv_rm3lineto, psw->len);
+                        *psw->dp = cmd_count_op(cmd_opv_rm3lineto, psw->len, psw->cldev->memory);
                         return 0;
                     }
                     break;
@@ -1201,6 +1283,8 @@ cmd_put_segment(cmd_segment_writer * psw, byte op,
 /* Put out a line segment command. */
 #define cmd_put_rmoveto(psw, operands)\
   cmd_put_segment(psw, cmd_opv_rmoveto, operands, sn_none)
+#define cmd_put_rgapto(psw, operands, notes)\
+  cmd_put_segment(psw, cmd_opv_rgapto, operands, notes)
 #define cmd_put_rlineto(psw, operands, notes)\
   cmd_put_segment(psw, cmd_opv_rlineto, operands, notes)
 
@@ -1281,9 +1365,9 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     struct { fixed vs[6]; } prev;
 
     first.x = first.y = out.x = out.y = start.x = start.y = 0; /* Quiet gcc warning. */
-    if_debug4('p', "[p]initial (%g,%g), clip [%g..%g)\n",
-              fixed2float(px), fixed2float(py),
-              fixed2float(ymin), fixed2float(ymax));
+    if_debug4m('p', cldev->memory, "[p]initial (%g,%g), clip [%g..%g)\n",
+               fixed2float(px), fixed2float(py),
+               fixed2float(ymin), fixed2float(ymax));
     gx_path_enum_init(&cenum, ppath);
     writer.cldev = cldev;
     writer.pcls = pcls;
@@ -1313,8 +1397,8 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                 /* All done. */
                 pcls->rect.x = fixed2int_var(px);
                 pcls->rect.y = fixed2int_var(py);
-                if_debug2('p', "[p]final (%d,%d)\n",
-                          pcls->rect.x, pcls->rect.y);
+                if_debug2m('p', cldev->memory, "[p]final (%d,%d)\n",
+                           pcls->rect.x, pcls->rect.y);
                 return set_cmd_put_op(dp, cldev, pcls, path_op, 1);
             case gs_pe_moveto:
                 /* If the path is open and needs an implicit close, */
@@ -1328,18 +1412,18 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                 start_skip = false;
                 if ((start_side = side = which_side(B)) != 0) {
                     out.x = A, out.y = B;
-                    if_debug3('p', "[p]skip moveto (%g,%g) side %d\n",
-                              fixed2float(out.x), fixed2float(out.y),
-                              side);
+                    if_debug3m('p', cldev->memory, "[p]skip moveto (%g,%g) side %d\n",
+                               fixed2float(out.x), fixed2float(out.y),
+                               side);
                     continue;
                 }
                 C = A - px, D = B - py;
                 first.x = px = A, first.y = py = B;
                 code = cmd_put_rmoveto(&writer, &C);
-                if_debug2('p', "[p]moveto (%g,%g)\n",
-                          fixed2float(px), fixed2float(py));
+                if_debug2m('p', cldev->memory, "[p]moveto (%g,%g)\n",
+                           fixed2float(px), fixed2float(py));
                 break;
-            case gs_pe_lineto:
+            case gs_pe_gapto:
                 {
                     int next_side = which_side(B);
                     segment_notes notes =
@@ -1350,9 +1434,9 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                             start_skip = true;
                         out.x = A, out.y = B;
                         out_notes = notes;
-                        if_debug3('p', "[p]skip lineto (%g,%g) side %d\n",
-                                  fixed2float(out.x), fixed2float(out.y),
-                                  side);
+                        if_debug3m('p', cldev->memory, "[p]skip gapto (%g,%g) side %d\n",
+                                   fixed2float(out.x), fixed2float(out.y),
+                                   side);
                         continue;
                     }
                     /* If we skipped any segments, put out a moveto/lineto. */
@@ -1366,9 +1450,51 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                         if (code < 0)
                             return code;
                         px = out.x, py = out.y;
-                        if_debug3('p', "[p]catchup %s (%g,%g) for line\n",
-                                  (open < 0 ? "moveto" : "lineto"),
-                                  fixed2float(px), fixed2float(py));
+                        if_debug3m('p', cldev->memory, "[p]catchup %s (%g,%g) for line\n",
+                                   (open < 0 ? "moveto" : "lineto"),
+                                   fixed2float(px), fixed2float(py));
+                    }
+                    if ((side = next_side) != 0) {	/* Note a vertex going outside the clip region. */
+                        out.x = A, out.y = B;
+                    }
+                    C = A - px, D = B - py;
+                    px = A, py = B;
+                    open = 1;
+                    code = cmd_put_rgapto(&writer, &C, notes);
+                }
+                if_debug3m('p', cldev->memory, "[p]gapto (%g,%g) side %d\n",
+                           fixed2float(px), fixed2float(py), side);
+                break;
+            case gs_pe_lineto:
+                {
+                    int next_side = which_side(B);
+                    segment_notes notes =
+                    gx_path_enum_notes(&cenum) & keep_notes;
+
+                    if (next_side == side && side != 0) {	/* Skip a line completely outside the clip region. */
+                        if (open < 0)
+                            start_skip = true;
+                        out.x = A, out.y = B;
+                        out_notes = notes;
+                        if_debug3m('p', cldev->memory, "[p]skip lineto (%g,%g) side %d\n",
+                                   fixed2float(out.x), fixed2float(out.y),
+                                   side);
+                        continue;
+                    }
+                    /* If we skipped any segments, put out a moveto/lineto. */
+                    if (side && (px != out.x || py != out.y || first_point())) {
+                        C = out.x - px, D = out.y - py;
+                        if (open < 0) {
+                            first = out;
+                            code = cmd_put_rmoveto(&writer, &C);
+                        } else
+                            code = cmd_put_rlineto(&writer, &C, out_notes);
+                        if (code < 0)
+                            return code;
+                        px = out.x, py = out.y;
+                        if_debug3m('p', cldev->memory, "[p]catchup %s (%g,%g) for line\n",
+                                   (open < 0 ? "moveto" : "lineto"),
+                                   fixed2float(px), fixed2float(py));
                     }
                     if ((side = next_side) != 0) {	/* Note a vertex going outside the clip region. */
                         out.x = A, out.y = B;
@@ -1378,8 +1504,8 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                     open = 1;
                     code = cmd_put_rlineto(&writer, &C, notes);
                 }
-                if_debug3('p', "[p]lineto (%g,%g) side %d\n",
-                          fixed2float(px), fixed2float(py), side);
+                if_debug3m('p', cldev->memory, "[p]lineto (%g,%g) side %d\n",
+                           fixed2float(px), fixed2float(py), side);
                 break;
             case gs_pe_closepath:
 #ifdef DEBUG
@@ -1394,8 +1520,9 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                         case gs_pe_moveto:
                             break;
                         default:
-                            lprintf1("closepath followed by %d, not end/moveto!\n",
-                                     op);
+                            mlprintf1(cldev->memory,
+                                      "closepath followed by %d, not end/moveto!\n",
+                                      op);
                     }
                 }
 #endif
@@ -1408,8 +1535,8 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                         if (code < 0)
                             return code;
                         px = out.x, py = out.y;
-                        if_debug2('p', "[p]catchup line (%g,%g) for close\n",
-                                  fixed2float(px), fixed2float(py));
+                        if_debug2m('p', cldev->memory, "[p]catchup line (%g,%g) for close\n",
+                                   fixed2float(px), fixed2float(py));
                     }
                     if (open > 0 && start_skip) {	/* Draw the closing line back to the start. */
                         C = start.x - px, D = start.y - py;
@@ -1417,8 +1544,8 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                         if (code < 0)
                             return code;
                         px = start.x, py = start.y;
-                        if_debug2('p', "[p]draw close to (%g,%g)\n",
-                                  fixed2float(px), fixed2float(py));
+                        if_debug2m('p', cldev->memory, "[p]draw close to (%g,%g)\n",
+                                   fixed2float(px), fixed2float(py));
                     }
                 }
                 /*
@@ -1450,7 +1577,7 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                 open = 0;
                 px = first.x, py = first.y;
                 code = cmd_put_segment(&writer, cmd_opv_closepath, &A, sn_none);
-                if_debug0('p', "[p]close\n");
+                if_debug0m('p', cldev->memory, "[p]close\n");
                 break;
             case gs_pe_curveto:
                 {
@@ -1477,9 +1604,10 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                                     start_skip = true;
                                 out.x = E, out.y = F;
                                 out_notes = notes;
-                                if_debug3('p', "[p]skip curveto (%g,%g) side %d\n",
-                                     fixed2float(out.x), fixed2float(out.y),
-                                          side);
+                                if_debug3m('p', cldev->memory,
+                                           "[p]skip curveto (%g,%g) side %d\n",
+                                           fixed2float(out.x), fixed2float(out.y),
+                                           side);
                                 continue;
                             }
                             out_side = all_side;
@@ -1498,9 +1626,10 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                             if (code < 0)
                                 return code;
                             px = out.x, py = out.y;
-                            if_debug3('p', "[p]catchup %s (%g,%g) for curve\n",
-                                      (open < 0 ? "moveto" : "lineto"),
-                                      fixed2float(px), fixed2float(py));
+                            if_debug3m('p', cldev->memory,
+                                       "[p]catchup %s (%g,%g) for curve\n",
+                                       (open < 0 ? "moveto" : "lineto"),
+                                       fixed2float(px), fixed2float(py));
                         }
                         if ((side = out_side) != 0) {	/* Note a vertex going outside the clip region. */
                             out.x = E, out.y = F;
@@ -1511,10 +1640,11 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                         const fixed *optr = vs;
                         byte op;
 
-                        if_debug7('p', "[p]curveto (%g,%g; %g,%g; %g,%g) side %d\n",
-                                  fixed2float(A), fixed2float(B),
-                                  fixed2float(C), fixed2float(D),
-                                  fixed2float(E), fixed2float(F), side);
+                        if_debug7m('p', cldev->memory,
+                                   "[p]curveto (%g,%g; %g,%g; %g,%g) side %d\n",
+                                   fixed2float(A), fixed2float(B),
+                                   fixed2float(C), fixed2float(D),
+                                   fixed2float(E), fixed2float(F), side);
                         E -= C, F -= D;
                         C -= A, D -= B;
                         A -= px, B -= py;
