@@ -39,6 +39,7 @@
 #include "gsnamecl.h"
 #include "gp.h"
 #include "gscms.h"
+#include "gxrplane.h"
 
 /* See Drivers.htm for documentation of the driver interface. */
 
@@ -236,16 +237,6 @@ typedef enum {
     GX_CINFO_SEP_LIN
 } gx_color_enc_sep_lin_t;
 
-/*
- * Color model component polarity. An "unknown" value has been added to
- * this enumeration.
- */
-typedef enum {
-    GX_CINFO_POLARITY_UNKNOWN = -1,
-    GX_CINFO_POLARITY_SUBTRACTIVE = 0,
-    GX_CINFO_POLARITY_ADDITIVE
-} gx_color_polarity_t;
-
 /*
  * Enumerator to indicate if a color model will support overprint mode.
  *
@@ -692,6 +683,38 @@ typedef struct gx_stroked_gradient_recognizer_s {
 typedef struct gx_device_cached_colors_s {
     gx_color_index black, white;
 } gx_device_cached_colors_t;
+
+/*
+ * Define the parameters controlling banding.
+ */
+/* if you make any additions/changes to this structure you need to make
+   the appropriate additions/changes to the compare_gdev_prn_space_params()
+   function in gdevprn.c */
+typedef struct gx_band_params_s {
+    int BandWidth;		/* (optional) band width in pixels */
+    int BandHeight;		/* (optional) */
+    long BandBufferSpace;	/* (optional) */
+} gx_band_params_t;
+
+#define BAND_PARAMS_INITIAL_VALUES 0, 0, 0
+
+typedef enum {
+    BandingAuto = 0,
+    BandingAlways,
+    BandingNever
+} gdev_banding_type;
+
+/* if you make any additions/changes to this structure you need to make
+   the appropriate additions/changes to the compare_gdev_prn_space_params()
+   function in gdevprn.c */
+typedef struct gdev_space_params_s {
+    long MaxBitmap;		/* max size of non-buffered bitmap */
+    long BufferSpace;		/* space to use for buffer */
+    gx_band_params_t band;	/* see gxband.h */
+    bool params_are_read_only;	/* true if put_params may not modify this struct */
+    gdev_banding_type banding_type;	/* used to force banding or bitmap */
+} gdev_space_params;
+
 #define gx_device_common\
         int params_size;		/* OBSOLETE if stype != 0: */\
                                         /* size of this structure */\
@@ -716,10 +739,13 @@ typedef struct gx_device_cached_colors_s {
         gx_device_cached_colors_t cached_colors;\
         int width;			/* width in pixels */\
         int height;			/* height in pixels */\
+        int pad;                        /* pad to use for buffers; 0 for default */\
+        int log2_align_mod;             /* align to use for buffers; 0 for default */\
+        int is_planar;                  /* 1 planar, 0 for chunky */\
         int LeadingEdge;                /* see below */\
         float MediaSize[2];		/* media dimensions in points */\
         float ImagingBBox[4];		/* imageable region in points */\
-          bool ImagingBBox_set;\
+        bool ImagingBBox_set;\
         float HWResolution[2];		/* resolution, dots per inch */\
         float MarginsHWResolution[2];	/* resolution for Margins */\
         float Margins[2];		/* offset of physical page corner */\
@@ -738,6 +764,8 @@ typedef struct gx_device_cached_colors_s {
         long band_offset_y;		/* for rendering that is phase sensitive (old wtsimdi) */\
         gx_stroked_gradient_recognizer_t sgr;\
         int MaxPatternBitmap;		/* Threshold for switching to pattern_clist mode */\
+        bool page_uses_transparency;    /* PDF 1.4 transparency is used. */\
+        gdev_space_params space_params;\
         cmm_dev_profile_t *icc_struct;  /* object dependent profiles */\
         gs_graphics_type_tag_t   graphics_type_tag;   /* e.g. vector, image or text */\
         gx_page_device_procs page_procs;       /* must be last */\
@@ -1173,7 +1201,6 @@ typedef struct gs_param_list_s gs_param_list;
     const gs_transparency_group_params_t *ptgp,\
     const gs_rect *pbbox,\
     gs_imager_state *pis,\
-    gs_transparency_state_t **ppts,\
     gs_memory_t *mem)
 #define dev_proc_begin_transparency_group(proc)\
   dev_t_proc_begin_transparency_group(proc, gx_device)
@@ -1187,8 +1214,7 @@ typedef struct gs_param_list_s gs_param_list;
 */
 #define dev_t_proc_end_transparency_group(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis,\
-    gs_transparency_state_t **ppts)
+    gs_imager_state *pis)
 #define dev_proc_end_transparency_group(proc)\
   dev_t_proc_end_transparency_group(proc, gx_device)
 
@@ -1202,7 +1228,6 @@ typedef struct gs_param_list_s gs_param_list;
     const gx_transparency_mask_params_t *ptmp,\
     const gs_rect *pbbox,\
     gs_imager_state *pis,\
-    gs_transparency_state_t **ppts,\
     gs_memory_t *mem)
 #define dev_proc_begin_transparency_mask(proc)\
   dev_t_proc_begin_transparency_mask(proc, gx_device)
@@ -1215,18 +1240,18 @@ typedef struct gs_param_list_s gs_param_list;
 */
 #define dev_t_proc_end_transparency_mask(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_imager_state *pis,\
-    gs_transparency_mask_t **pptm)
+    gs_imager_state *pis)
 #define dev_proc_end_transparency_mask(proc)\
   dev_t_proc_end_transparency_mask(proc, gx_device)
 
 /*
-  Pop the transparency stack, discarding the top element, which may be
-  either a group or a mask.  Set *ppts to 0 iff the stack is now empty.
+  This will clean up the entire device allocations as something went
+  wrong in the middle of reading in the source content while we are dealing with
+  a transparency device.
 */
 #define dev_t_proc_discard_transparency_layer(proc, dev_t)\
   int proc(gx_device *dev,\
-    gs_transparency_state_t **ppts)
+    gs_imager_state *pis)
 #define dev_proc_discard_transparency_layer(proc)\
   dev_t_proc_discard_transparency_layer(proc, gx_device)
 
@@ -1479,6 +1504,27 @@ typedef struct gs_devn_params_s gs_devn_params;
 #define dev_proc_copy_alpha_hl_color(proc)\
   dev_t_proc_copy_alpha_hl_color(proc, gx_device)
 
+typedef struct gx_process_page_options_s gx_process_page_options_t;
+
+struct gx_process_page_options_s
+{
+    int (*init_buffer_fn)(void *arg, gx_device *dev, gs_memory_t *memory, int w, int h, void **buffer);
+    void (*free_buffer_fn)(void *arg, gx_device *dev, gs_memory_t *memory, void *buffer);
+    int (*process_fn)(void *arg, gx_device *dev, gx_device *bdev, const gs_int_rect *rect, void *buffer);
+    int (*output_fn)(void *arg, gx_device *dev, void *buffer);
+    void *arg;
+    int options; /* A mask of GX_PROCPAGE_... options bits */
+};
+
+/* If GX_PROCPAGE_BOTTOM_UP, then we run from band n-1 to band 0, rather than
+ * 0 to n-1. */
+#define GX_PROCPAGE_BOTTOM_UP 1
+
+#define dev_t_proc_process_page(proc, dev_t)\
+  int proc(dev_t *dev, gx_process_page_options_t *options)
+#define dev_proc_process_page(proc)\
+  dev_t_proc_process_page(proc, gx_device)
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -1554,6 +1600,7 @@ typedef struct gs_devn_params_s gs_devn_params;
         dev_t_proc_strip_copy_rop2((*strip_copy_rop2), dev_t);\
         dev_t_proc_strip_tile_rect_devn((*strip_tile_rect_devn), dev_t);\
         dev_t_proc_copy_alpha_hl_color((*copy_alpha_hl_color), dev_t);\
+        dev_t_proc_process_page((*process_page), dev_t);\
 }
 
 /*
@@ -1706,6 +1753,14 @@ extern_st(st_device_null);
 void gx_device_init(gx_device * dev, const gx_device * proto,
                     gs_memory_t * mem, bool internal);
 
+/*
+ * Identical to gx_device_init, except that the reference counting is set
+ * up so that it doesn't attempt to free the device structure when the last
+ * instance is removed, and the device is always internal (never retained).
+ */
+void gx_device_init_on_stack(gx_device * dev, const gx_device * proto,
+                             gs_memory_t * mem);
+
 /* Make a null device. */
 /* The gs_memory_t argument is 0 if the device is temporary and local, */
 /* or the allocator that was used to allocate it if it is a real object. */
@@ -1721,9 +1776,17 @@ void gx_device_set_target(gx_device_forward *fdev, gx_device *target);
 void gx_device_retain(gx_device *dev, bool retained);
 
 /* Calculate the raster (number of bytes in a scan line), */
-/* with byte or word padding. */
+/* with byte or device padding. */
 uint gx_device_raster(const gx_device * dev, bool pad_to_word);
 
+/* Calculate the raster (number of bytes in a scan line), */
+/* with byte or device padding forcing chunky format. */
+uint gx_device_raster_chunky(const gx_device * dev, bool pad);
+
+/* Calculate the raster (with device padding) optionally for a given
+ * render_plane (may be NULL). */
+uint gx_device_raster_plane(const gx_device * dev, const gx_render_plane_t *render_plane);
+
 /* Adjust the resolution for devices that only have a fixed set of */
 /* geometries, so that the apparent size in inches remains constant. */
 /* If fit=1, the resolution is adjusted so that the entire image fits; */
@@ -1739,10 +1802,10 @@ void gx_device_set_margins(gx_device * dev, const float *margins /*[4] */ ,
 void gx_device_set_width_height(gx_device * dev, int width, int height);
 
 /* Set the resolution (in pixels per inch), updating width and height. */
-void gx_device_set_resolution(gx_device * dev, floatp x_dpi, floatp y_dpi);
+void gx_device_set_resolution(gx_device * dev, double x_dpi, double y_dpi);
 
 /* Set the MediaSize (in 1/72" units), updating width and height. */
-void gx_device_set_media_size(gx_device * dev, floatp media_width, floatp media_height);
+void gx_device_set_media_size(gx_device * dev, double media_width, double media_height);
 
 /****** BACKWARD COMPATIBILITY ******/
 #define gx_device_set_page_size(dev, w, h)\
