@@ -320,7 +320,7 @@ static void down_core_2(gx_downscaler_t *ds,
     {
         /* Right to Left pass (no min feature size) */
         errors += awidth;
-        inp += awidth*2-1;
+        inp += (awidth-1)*2;
         outp = inp;
         for (x = awidth; x > 0; x--)
         {
@@ -415,7 +415,7 @@ static void down_core_3(gx_downscaler_t *ds,
     {
         /* Right to Left pass (no min feature size) */
         errors += awidth;
-        inp += awidth*3-1;
+        inp += (awidth-1)*3;
         outp = inp;
         for (x = awidth; x > 0; x--)
         {
@@ -514,7 +514,7 @@ static void down_core_4(gx_downscaler_t *ds,
     {
         /* Right to Left pass (no min feature size) */
         errors += awidth;
-        inp += awidth*4-1;
+        inp += (awidth-1)*4;
         outp = inp;
         for (x = awidth; x > 0; x--)
         {
@@ -712,6 +712,300 @@ static void down_core_mfs(gx_downscaler_t *ds,
     pack_8to1(out_buffer, outp, awidth);
 }
 
+/* CMYK 32 -> 4bit core */
+static void down_core4(gx_downscaler_t *ds,
+                       byte            *out_buffer,
+                       byte            *in_buffer,
+                       int              row,
+                       int              plane /* unused */,
+                       int              span)
+{
+    int        x, xx, y, value, comp;
+    int        e_downleft, e_down, e_forward = 0;
+    int        pad_white;
+    byte      *inp, *outp;
+    int        width     = ds->width;
+    int        awidth    = ds->awidth;
+    int        factor    = ds->factor;
+    int       *errors    = ds->errors;
+    const int  threshold = factor*factor*128;
+    const int  max_value = factor*factor*255;
+
+    pad_white = (awidth - width) * factor * 4;
+    if (pad_white < 0)
+        pad_white = 0;
+
+    if (pad_white)
+    {
+        inp = in_buffer + width*factor * 4;
+        for (y = factor; y > 0; y--)
+        {
+            memset(inp, 0xFF, pad_white);
+            inp += span;
+        }
+    }
+
+    if ((row & 1) == 0)
+    {
+        /* Left to Right pass (no min feature size) */
+        const int back = span * factor - 4;
+        for (comp = 0; comp < 4; comp++)
+        {
+            errors = ds->errors + (awidth+3)*comp + 2;
+            inp = in_buffer + comp;
+            outp = inp;
+            for (x = awidth; x > 0; x--)
+            {
+                value = e_forward + *errors;
+                for (xx = factor; xx > 0; xx--)
+                {
+                    for (y = factor; y > 0; y--)
+                    {
+                        value += *inp;
+                        inp += span;
+                    }
+                    inp -= back;
+                }
+                if (value >= threshold)
+                {
+                    *outp = 1; outp += 4;
+                    value -= max_value;
+                }
+                else
+                {
+                    *outp = 0; outp += 4;
+                }
+                e_forward  = value * 7/16;
+                e_downleft = value * 3/16;
+                e_down     = value * 5/16;
+                value     -= e_forward + e_downleft + e_down;
+                errors[-2] += e_downleft;
+                errors[-1] += e_down;
+                *errors++   = value;
+            }
+        }
+        outp = in_buffer;
+    }
+    else
+    {
+        /* Right to Left pass (no min feature size) */
+        const int back = span * factor + 4;
+        for (comp = 0; comp < 4; comp++)
+        {
+            errors = ds->errors + (awidth+3)*comp + awidth;
+            inp = in_buffer + awidth*factor*4 - 4 + comp;
+            outp = inp;
+            for (x = awidth; x > 0; x--)
+            {
+                value = e_forward + *errors;
+                for (xx = factor; xx > 0; xx--)
+                {
+                    for (y = factor; y > 0; y--)
+                    {
+                        value += *inp;
+                        inp += span;
+                    }
+                    inp -= back;
+                }
+                if (value >= threshold)
+                {
+                    *outp = 1; outp -= 4;
+                    value -= max_value;
+                }
+                else
+                {
+                    *outp = 0; outp -= 4;
+                }
+                e_forward  = value * 7/16;
+                e_downleft = value * 3/16;
+                e_down     = value * 5/16;
+                value     -= e_forward + e_downleft + e_down;
+                errors[2] += e_downleft;
+                errors[1] += e_down;
+                *errors--   = value;
+            }
+        }
+        outp = in_buffer + awidth*factor*4 - (awidth*4);
+    }
+    pack_8to1(out_buffer, outp, awidth*4);
+}
+
+static void down_core4_mfs(gx_downscaler_t *ds,
+                           byte            *out_buffer,
+                           byte            *in_buffer,
+                           int              row,
+                           int              plane /* unused */,
+                           int              span)
+{
+    int        x, xx, y, value, comp;
+    int        e_downleft, e_down, e_forward = 0;
+    int        pad_white;
+    byte      *inp, *outp;
+    int        width     = ds->width;
+    int        awidth    = ds->awidth;
+    int        factor    = ds->factor;
+    int       *errors;
+    byte      *mfs_data;
+    const int  threshold = factor*factor*128;
+    const int  max_value = factor*factor*255;
+
+    pad_white = (awidth - width) * factor * 4;
+    if (pad_white < 0)
+        pad_white = 0;
+
+    if (pad_white)
+    {
+        inp = in_buffer + width*factor*4;
+        for (y = factor*4; y > 0; y--)
+        {
+            memset(inp, 0xFF, pad_white);
+            inp += span;
+        }
+    }
+
+    inp = in_buffer;
+    if ((row & 1) == 0)
+    {
+        /* Left to Right pass (with min feature size = 2) */
+        const int back = span * factor - 4;
+        byte *outp_base = inp;
+        for (comp = 0; comp < 4; comp++)
+        {
+            byte mfs, force_forward = 0;
+            errors = ds->errors + (awidth+3)*comp + 2;
+            inp = outp_base;
+            outp = outp_base++;
+            mfs_data = ds->mfs_data + (awidth+1)*comp;
+            *mfs_data++ = mfs_clear;
+            for (x = awidth; x > 0; x--)
+            {
+                value = e_forward + *errors;
+                for (xx = factor; xx > 0; xx--)
+                {
+                    for (y = factor; y > 0; y--)
+                    {
+                        value += *inp;
+                        inp += span;
+                    }
+                    inp -= back;
+                }
+                mfs = *mfs_data;
+                *mfs_data++ = mfs_clear;
+                if ((mfs & mfs_force_off) || force_forward)
+                {
+                    /* We are being forced to be 0 */
+                    *outp = 0; outp += 4;
+                    force_forward = 0;
+                }
+                else if (value < threshold)
+                {
+                    /* We want to be 0 anyway */
+                    *outp = 0; outp += 4;
+                    if ((mfs & (mfs_above_is_0 | mfs_above_left_is_0))
+                            != (mfs_above_is_0 | mfs_above_left_is_0))
+                    {
+                        /* We aren't in a group anyway, so must force other
+                         * pixels. */
+                        mfs_data[-2] |= mfs_force_off;
+                        mfs_data[-1] |= mfs_force_off;
+                        force_forward = 1;
+                    }
+                    else
+                    {
+                        /* No forcing, but we need to tell other pixels that
+                         * we were 0. */
+                        mfs_data[-2] |= mfs_above_is_0;
+                        mfs_data[-1] |= mfs_above_left_is_0;
+                    }
+                }
+                else
+                {
+                    *outp = 1; outp += 4;
+                    value -= max_value;
+                }
+                e_forward  = value * 7/16;
+                e_downleft = value * 3/16;
+                e_down     = value * 5/16;
+                value     -= e_forward + e_downleft + e_down;
+                errors[-2] += e_downleft;
+                errors[-1] += e_down;
+                *errors++   = value;
+            }
+        }
+        outp = outp_base-4;
+    }
+    else
+    {
+        /* Right to Left pass (with min feature size = 2) */
+        const int back = span * factor + 4;
+        byte *outp_base = inp + awidth*factor - 4;
+        for (comp = 0; comp < 4; comp++)
+        {
+            byte mfs, force_forward = 0;
+            errors = ds->errors + (awidth+3)*comp + awidth;
+            mfs_data = ds->mfs_data + (awidth+1)*comp + awidth;
+            inp = outp_base;
+            outp = outp_base++;
+            *mfs_data-- = 0;
+            for (x = awidth; x > 0; x--)
+            {
+                value = e_forward + *errors;
+                for (xx = factor; xx > 0; xx--)
+                {
+                    for (y = factor; y > 0; y--)
+                    {
+                        value += *inp;
+                        inp += span;
+                    }
+                    inp -= back;
+                }
+                mfs = *mfs_data;
+                *mfs_data-- = mfs_clear;
+                if ((mfs & mfs_force_off) || force_forward)
+                {
+                    /* We are being forced to be 0 */
+                    *outp = 0; outp -= 4;
+                    force_forward = 0;
+                }
+                else if (value < threshold)
+                {
+                    *outp = 0; outp -= 4;
+                    if ((mfs & (mfs_above_is_0 | mfs_above_left_is_0))
+                            != (mfs_above_is_0 | mfs_above_left_is_0))
+                    {
+                        /* We aren't in a group anyway, so must force other
+                         * pixels. */
+                        mfs_data[1] |= mfs_force_off;
+                        mfs_data[2] |= mfs_force_off;
+                        force_forward = 1;
+                    }
+                    else
+                    {
+                        /* No forcing, but we need to tell other pixels that
+                         * we were 0. */
+                        mfs_data[1] |= mfs_above_is_0;
+                        mfs_data[2] |= mfs_above_left_is_0;
+                    }
+                }
+                else
+                {
+                    *outp = 1; outp -= 4;
+                    value -= max_value;
+                }
+                e_forward  = value * 7/16;
+                e_downleft = value * 3/16;
+                e_down     = value * 5/16;
+                value     -= e_forward + e_downleft + e_down;
+                errors[2] += e_downleft;
+                errors[1] += e_down;
+                *errors--   = value;
+            }
+        }
+        outp = outp_base - awidth*factor;
+    }
+    pack_8to1(out_buffer, outp, awidth*4);
+}
+
 /* Grey (or planar) downscale code */
 static void down_core16(gx_downscaler_t *ds,
                         byte            *outp,
@@ -976,11 +1270,11 @@ static void down_core8_3_2(gx_downscaler_t *ds,
         int g = inp[2*span+0];
         int h = inp[2*span+1];
         int i = inp[2*span+2];
-	outp[0      ] = (4*a+2*b+2*d+e+4)/9;
-	outp[1      ] = (4*c+2*b+2*f+e+4)/9;
-	outp[dspan+0] = (4*g+2*h+2*d+e+4)/9;
-	outp[dspan+1] = (4*i+2*h+2*f+e+4)/9;
-	outp += 2;
+        outp[0      ] = (4*a+2*b+2*d+e+4)/9;
+        outp[1      ] = (4*c+2*b+2*f+e+4)/9;
+        outp[dspan+0] = (4*g+2*h+2*d+e+4)/9;
+        outp[dspan+1] = (4*i+2*h+2*f+e+4)/9;
+        outp += 2;
         inp += 3;
     }
 }
@@ -1027,23 +1321,23 @@ static void down_core8_3_4(gx_downscaler_t *ds,
         int g = inp[2*span+0];
         int h = inp[2*span+1];
         int i = inp[2*span+2];
-	outp[        0] = a;
-	outp[        1] = (a+2*b+1)/3;
-	outp[        2] = (c+2*b+1)/3;
-	outp[        3] = c;
-	outp[  dspan+0] = (a+2*d+1)/3;
-	outp[  dspan+1] = (a+2*b+2*d+4*e+3)/9;
-	outp[  dspan+2] = (c+2*b+2*f+4*e+3)/9;
-	outp[  dspan+3] = (c+2*f+1)/3;
-	outp[2*dspan+0] = (g+2*d+1)/3;
-	outp[2*dspan+1] = (g+2*h+2*d+4*e+3)/9;
-	outp[2*dspan+2] = (i+2*h+2*f+4*e+3)/9;
-	outp[2*dspan+3] = (i+2*f+1)/3;
-	outp[3*dspan+0] = g;
-	outp[3*dspan+1] = (g+2*h+1)/3;
-	outp[3*dspan+2] = (i+2*h+1)/3;
-	outp[3*dspan+3] = i;
-	outp += 4;
+        outp[        0] = a;
+        outp[        1] = (a+2*b+1)/3;
+        outp[        2] = (c+2*b+1)/3;
+        outp[        3] = c;
+        outp[  dspan+0] = (a+2*d+1)/3;
+        outp[  dspan+1] = (a+2*b+2*d+4*e+3)/9;
+        outp[  dspan+2] = (c+2*b+2*f+4*e+3)/9;
+        outp[  dspan+3] = (c+2*f+1)/3;
+        outp[2*dspan+0] = (g+2*d+1)/3;
+        outp[2*dspan+1] = (g+2*h+2*d+4*e+3)/9;
+        outp[2*dspan+2] = (i+2*h+2*f+4*e+3)/9;
+        outp[2*dspan+3] = (i+2*f+1)/3;
+        outp[3*dspan+0] = g;
+        outp[3*dspan+1] = (g+2*h+1)/3;
+        outp[3*dspan+2] = (i+2*h+1)/3;
+        outp[3*dspan+3] = i;
+        outp += 4;
         inp += 3;
     }
 }
@@ -1129,6 +1423,100 @@ static void down_core24(gx_downscaler_t *ds,
     }
 }
 
+/* CMYK downscale (no error diffusion) code */
+
+static void down_core32(gx_downscaler_t *ds,
+                        byte            *outp,
+                        byte            *in_buffer,
+                        int              row,
+                        int              plane,
+                        int              span)
+{
+    int   x, xx, y, value;
+    int   pad_white;
+    byte *inp;
+    int   width  = ds->width;
+    int   awidth = ds->awidth;
+    int   factor = ds->factor;
+    int   div    = factor*factor;
+
+    pad_white = (awidth - width) * factor * 4;
+    if (pad_white < 0)
+        pad_white = 0;
+
+    if (pad_white)
+    {
+        inp = in_buffer + width*factor*4;
+        for (y = factor; y > 0; y--)
+        {
+            memset(inp, 0xFF, pad_white);
+            inp += span;
+        }
+    }
+
+    inp = in_buffer;
+    {
+        /* Left to Right pass (no min feature size) */
+        const int back  = span * factor - 4;
+        const int back2 = factor * 4 - 1;
+        for (x = awidth; x > 0; x--)
+        {
+            /* C */
+            value = 0;
+            for (xx = factor; xx > 0; xx--)
+            {
+                for (y = factor; y > 0; y--)
+                {
+                    value += *inp;
+                    inp += span;
+                }
+                inp -= back;
+            }
+            inp -= back2;
+            *outp++ = (value+(div>>1))/div;
+            /* M */
+            value = 0;
+            for (xx = factor; xx > 0; xx--)
+            {
+                for (y = factor; y > 0; y--)
+                {
+                    value += *inp;
+                    inp += span;
+                }
+                inp -= back;
+            }
+            inp -= back2;
+            *outp++ = (value+(div>>1))/div;
+            /* Y */
+            value = 0;
+            for (xx = factor; xx > 0; xx--)
+            {
+                for (y = factor; y > 0; y--)
+                {
+                    value += *inp;
+                    inp += span;
+                }
+                inp -= back;
+            }
+            inp -= back2;
+            *outp++ = (value+(div>>1))/div;
+            /* K */
+            value = 0;
+            for (xx = factor; xx > 0; xx--)
+            {
+                for (y = factor; y > 0; y--)
+                {
+                    value += *inp;
+                    inp += span;
+                }
+                inp -= back;
+            }
+            inp -= 3;
+            *outp++ = (value+(div>>1))/div;
+        }
+    }
+}
+
 static void decode_factor(int factor, int *up, int *down)
 {
     if (factor == 32)
@@ -1148,6 +1536,23 @@ gx_downscaler_scale(int width, int factor)
     return (width*up)/down;
 }
 
+int gx_downscaler_adjust_bandheight(int factor, int band_height)
+{
+    int up, down;
+
+    decode_factor(factor, &up, &down);
+    return (band_height/down)*down;
+}
+
+int
+gx_downscaler_scale_rounded(int width, int factor)
+{
+    int up, down;
+
+    decode_factor(factor, &up, &down);
+    return (width*up + down-1)/down;
+}
+
 int gx_downscaler_init_planar(gx_downscaler_t      *ds,
                               gx_device            *dev,
                               gs_get_bits_params_t *params,
@@ -1185,15 +1590,19 @@ int gx_downscaler_init_planar(gx_downscaler_t      *ds,
     {
         ds->params.data[i] = gs_alloc_bytes(dev->memory, span * downfactor,
                                             "gx_downscaler(planar_data)");
-        if (ds->params.data[i] == NULL)
+        if (ds->params.data[i] == NULL) {
+            code = gs_note_error(gs_error_VMerror);
             goto cleanup;
+        }
     }
-    if (upfactor)
+    if (upfactor > 1)
     {
         ds->scaled_data = gs_alloc_bytes(dev->memory, ds->scaled_span * upfactor * num_comps,
                                          "gx_downscaler(scaled_data)");
-        if (ds->scaled_data == NULL)
+        if (ds->scaled_data == NULL) {
+            code = gs_note_error(gs_error_VMerror);
             goto cleanup;
+        }
     }
 
     if ((src_bpc == 8) && (dst_bpc == 8) && (factor == 32))
@@ -1253,8 +1662,7 @@ int gx_downscaler_init_planar(gx_downscaler_t      *ds,
         ds->errors = (int *)gs_alloc_bytes(dev->memory,
                                            num_comps*(width+3)*sizeof(int),
                                            "gx_downscaler(errors)");
-        if (ds->errors == NULL)
-        {
+        if (ds->errors == NULL) {
             code = gs_note_error(gs_error_VMerror);
             goto cleanup;
         }
@@ -1320,6 +1728,13 @@ int gx_downscaler_init(gx_downscaler_t   *ds,
     {
         core = &down_core16;
     }
+    else if ((src_bpc == 8) && (dst_bpc == 1) && (num_comps == 4))
+    {
+        if (mfs > 1)
+            core = &down_core4_mfs;
+        else
+            core = &down_core4;
+    }
     else if ((src_bpc == 8) && (dst_bpc == 1) && (num_comps == 1))
     {
         if (mfs > 1)
@@ -1335,7 +1750,7 @@ int gx_downscaler_init(gx_downscaler_t   *ds,
         else
             core = &down_core;
     }
-    else if (factor == 1)
+    else if ((factor == 1) && (src_bpc == dst_bpc))
         core = NULL;
     else if ((src_bpc == 8) && (dst_bpc == 8) && (num_comps == 1))
     {
@@ -1350,6 +1765,8 @@ int gx_downscaler_init(gx_downscaler_t   *ds,
     }
     else if ((src_bpc == 8) && (dst_bpc == 8) && (num_comps == 3))
         core = &down_core24;
+    else if ((src_bpc == 8) && (dst_bpc == 8) && (num_comps == 4))
+         core = &down_core32;
     else {
         code = gs_note_error(gs_error_rangecheck);
         goto cleanup;
@@ -1465,9 +1882,9 @@ int gx_downscaler_get_bits_rectangle(gx_downscaler_t      *ds,
         /* Just copy a previous row from our stored buffer */
         for (plane=0; plane < ds->num_planes; plane++)
         {
-	    params->data[plane] = ds->scaled_data + (upfactor * plane + subrow) * ds->scaled_span;
+            params->data[plane] = ds->scaled_data + (upfactor * plane + subrow) * ds->scaled_span;
         }
-	return 0;
+        return 0;
     }
 
     rect.p.x = 0;
@@ -1528,7 +1945,7 @@ int gx_downscaler_get_bits_rectangle(gx_downscaler_t      *ds,
         {
             byte *scaled = ds->scaled_data + upfactor * plane * ds->scaled_span;
             (ds->down_core)(ds, scaled, params2.data[plane], row, plane, params2.raster);
-	    params->data[plane] = scaled;
+            params->data[plane] = scaled;
         }
     }
     else
@@ -1561,3 +1978,205 @@ gx_downscaler_copy_scan_lines(gx_downscaler_t *ds, int y,
     }
     return count;
 }
+
+typedef struct downscaler_process_page_arg_s
+{
+    gx_process_page_options_t *orig_options;
+    int upfactor;
+    int downfactor;
+    gx_downscaler_t ds;
+}
+downscaler_process_page_arg_t;
+
+typedef struct downscaler_process_page_buffer_s
+{
+    gx_device *bdev;
+    void *orig_buffer;
+}
+downscaler_process_page_buffer_t;
+
+static int downscaler_init_fn(void *arg_, gx_device *dev, gs_memory_t *memory, int w, int h, void **pbuffer)
+{
+    downscaler_process_page_arg_t *arg = (downscaler_process_page_arg_t *)arg_;
+    downscaler_process_page_buffer_t *buffer;
+    int code = 0;
+
+    buffer = (downscaler_process_page_buffer_t *)gs_alloc_bytes(memory, sizeof(*buffer), "downscaler process_page buffer");
+    if (buffer == NULL)
+        return gs_error_VMerror;
+    memset(buffer, 0, sizeof(*buffer));
+
+    if (arg->upfactor > arg->downfactor) {
+        code = gx_default_create_buf_device(&buffer->bdev, dev,
+                          (h*arg->upfactor + arg->downfactor-1)/arg->downfactor,
+                          NULL, memory, NULL);
+        if (code < 0) {
+            gs_free_object(memory, buffer, "downscaler process_page buffer");
+            return code;
+        }
+    }
+
+    if (arg->orig_options && arg->orig_options->init_buffer_fn) {
+        code = arg->orig_options->init_buffer_fn(arg->orig_options->arg, dev, memory,
+                                                 (w * arg->upfactor + arg->downfactor-1)/arg->downfactor,
+                                                 (h * arg->upfactor + arg->downfactor-1)/arg->downfactor,
+                                                 &buffer->orig_buffer);
+        if (code < 0) {
+            if (buffer->bdev)
+                dev_proc(dev, close_device)(dev);
+            gs_free_object(memory, buffer, "downscaler process_page buffer");
+            return code;
+        }
+    }
+
+    *pbuffer = (void *)buffer;
+    return code;
+}
+
+static int downscaler_process_fn(void *arg_, gx_device *dev, gx_device *bdev, const gs_int_rect *rect, void *buffer_)
+{
+    downscaler_process_page_arg_t *arg = (downscaler_process_page_arg_t *)arg_;
+    downscaler_process_page_buffer_t *buffer = (downscaler_process_page_buffer_t *)buffer_;
+    int code, raster_in, raster_out;
+    gs_get_bits_params_t params;
+    gs_int_rect in_rect, out_rect;
+    byte *in_ptr, *out_ptr;
+
+    in_rect.p.x = 0;
+    in_rect.p.y = 0;
+    in_rect.q.x = rect->q.x - rect->p.x;
+    in_rect.q.y = rect->q.y - rect->p.y;
+    out_rect.p.x = 0;
+    out_rect.p.y = 0;
+    out_rect.q.x = (in_rect.q.x * arg->upfactor + arg->downfactor-1) / arg->downfactor;
+    out_rect.q.y = (in_rect.q.y * arg->upfactor + arg->downfactor-1) / arg->downfactor;
+
+    /* Where do we get the data from? */
+    params.options = GB_COLORS_NATIVE | GB_ALPHA_NONE | GB_PACKING_CHUNKY | GB_RETURN_POINTER | GB_ALIGN_ANY | GB_OFFSET_0 | GB_RASTER_ANY;
+    code = dev_proc(bdev, get_bits_rectangle)(bdev, &in_rect, &params, NULL);
+    if (code < 0)
+        return code;
+    raster_in = params.raster;
+    in_ptr = params.data[0];
+
+    /* Where do we write it to? */
+    if (buffer->bdev) {
+        code = dev_proc(bdev, get_bits_rectangle)(buffer->bdev, &out_rect, &params, NULL);
+        if (code < 0)
+            return code;
+        raster_out = params.raster;
+        out_ptr = params.data[0];
+    } else {
+        raster_out = raster_in;
+        out_ptr = params.data[0];
+    }
+
+    /* Do the downscale */
+    if (arg->ds.down_core) {
+        int y;
+        for (y = rect->p.y; y < rect->q.y; y += arg->downfactor)
+        {
+            arg->ds.down_core(&arg->ds, out_ptr, in_ptr, y, 0, arg->ds.span);
+            in_ptr += arg->ds.span * arg->downfactor;
+            out_ptr += raster_out * arg->upfactor;
+        }
+    }
+
+    /* Pass on to further processing */
+    if (code >= 0 && arg->orig_options && arg->orig_options->process_fn) {
+        out_rect.p.y = rect->p.y*arg->upfactor/arg->downfactor;
+        out_rect.q.y += out_rect.p.y;
+        code = arg->orig_options->process_fn(arg->orig_options->arg, dev,
+                                             (buffer->bdev ? buffer->bdev : bdev),
+                                             &out_rect, buffer->orig_buffer);
+    }
+    return code;
+}
+
+static void
+downscaler_free_fn(void *arg_, gx_device *dev, gs_memory_t *memory, void *buffer_)
+{
+    downscaler_process_page_arg_t *arg = (downscaler_process_page_arg_t *)arg_;
+    downscaler_process_page_buffer_t *buffer = (downscaler_process_page_buffer_t *)buffer_;
+
+    arg->orig_options->free_buffer_fn(arg->orig_options->arg, dev, memory,
+                                      buffer->orig_buffer);
+    if (buffer->bdev)
+        dev_proc(dev, close_device)(dev);
+    gs_free_object(memory, buffer, "downscaler process_page buffer");
+}
+
+static int
+downscaler_output_fn(void *arg_, gx_device *dev, void *buffer_)
+{
+    downscaler_process_page_arg_t *arg = (downscaler_process_page_arg_t *)arg_;
+    downscaler_process_page_buffer_t *buffer = (downscaler_process_page_buffer_t *)buffer_;
+
+    return arg->orig_options->output_fn(arg->orig_options->arg, dev,
+                                        buffer->orig_buffer);
+}
+
+/* No error diffusion with process_page as bands need to be handled
+ * separately. */
+int gx_downscaler_process_page(gx_device                 *dev,
+                               gx_process_page_options_t *options,
+                               int                        factor)
+{
+    downscaler_process_page_arg_t arg = { 0 };
+    gx_process_page_options_t my_options = { 0 };
+    int num_comps = dev->color_info.num_components;
+    int src_bpc = dev->color_info.comp_bits[0];
+    int scaled_w;
+    gx_downscale_core *core;
+
+    arg.orig_options = options;
+    decode_factor(factor, &arg.upfactor, &arg.downfactor);
+    arg.ds.dev = dev;
+    arg.ds.width = (dev->width * arg.upfactor + arg.downfactor-1)/arg.downfactor;
+    arg.ds.awidth = arg.ds.width;
+    arg.ds.span = bitmap_raster(dev->width * num_comps * src_bpc);
+    scaled_w = (dev->width * arg.upfactor + arg.downfactor-1)/arg.downfactor;
+    arg.ds.factor = factor;
+    arg.ds.src_bpc = src_bpc;
+    arg.ds.scaled_span = bitmap_raster(scaled_w * num_comps * src_bpc);
+    arg.ds.num_planes = 0;
+
+    /* Choose an appropriate core */
+    if (factor > 8)
+    {
+        return gs_note_error(gs_error_rangecheck);
+    }
+    else if ((src_bpc == 16) && (num_comps == 1))
+    {
+        core = &down_core16;
+    }
+    else if (factor == 1)
+        core = NULL;
+    else if ((src_bpc == 8) && (num_comps == 1))
+    {
+        if (factor == 4)
+            core = &down_core8_4;
+        else if (factor == 3)
+            core = &down_core8_3;
+        else if (factor == 2)
+            core = &down_core8_2;
+        else
+            core = &down_core8;
+    }
+    else if ((src_bpc == 8) && (num_comps == 3))
+        core = &down_core24;
+    else if ((src_bpc == 8) && (num_comps == 4))
+         core = &down_core32;
+    else {
+        return gs_note_error(gs_error_rangecheck);
+    }
+    arg.ds.down_core = core;
+
+    my_options.init_buffer_fn = downscaler_init_fn;
+    my_options.process_fn = downscaler_process_fn;
+    my_options.output_fn = downscaler_output_fn;
+    my_options.free_buffer_fn = downscaler_free_fn;
+    my_options.arg = &arg;
+
+    return dev_proc(dev, process_page)(dev, &my_options);
+}
