@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Definitions for CID-keyed fonts */
 
 #ifndef gxfcid_INCLUDED
@@ -32,6 +34,7 @@
 typedef struct gs_font_cid_data_s {
     gs_cid_system_info_t CIDSystemInfo;
     int CIDCount;
+    int MaxCID;                 /* For font with GlyphDirectory highest CID != CIDCount. */
     int GDBytes;		/* optional, for standard glyph_data */
     /*int PaintType;*/
     /*float StrokeWidth;*/
