@@ -74,8 +74,8 @@ enum {
 /* Forward references */
 static int dict_create_contents(uint size, const ref * pdref, bool pack);
 
-/* Debugging statistics */
-#ifdef DEBUG
+/* Debugging statistics - uses a static, so not threadsafe. */
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
 struct stats_dict_s {
     long lookups;		/* total lookups */
     long probe1;		/* successful lookups on only 1 probe */
@@ -502,8 +502,8 @@ dict_put(ref * pdref /* t_dictionary */ , const ref * pkey, const ref * pvalue,
         } else {
             ref *kp = pdict->keys.value.refs + index;
 
-            if_debug2('d', "[d]0x%lx: fill key at 0x%lx\n",
-                      (ulong) pdict, (ulong) kp);
+            if_debug2m('d', (const gs_memory_t *)mem, "[d]0x%lx: fill key at 0x%lx\n",
+                       (ulong) pdict, (ulong) kp);
             store_check_dest(pdref, pkey);
             ref_assign_old_in(mem, &pdict->keys, kp, pkey,
                               "dict_put(key)");	/* set key of pair */
@@ -517,21 +517,22 @@ dict_put(ref * pdref /* t_dictionary */ , const ref * pkey, const ref * pvalue,
             if (pname->pvalue == pv_no_defn &&
                 CAN_SET_PVALUE_CACHE(pds, pdref, mem)
                 ) {		/* Set the cache. */
-                if_debug0('d', "[d]set cache\n");
+                if_debug0m('d', (const gs_memory_t *)mem, "[d]set cache\n");
                 pname->pvalue = pvslot;
             } else {		/* The cache can't be used. */
-                if_debug0('d', "[d]no cache\n");
+                if_debug0m('d', (const gs_memory_t *)mem, "[d]no cache\n");
                 pname->pvalue = pv_other;
             }
         }
         rcode = 1;
     }
-    if_debug8('d', "[d]0x%lx: put key 0x%lx 0x%lx\n  value at 0x%lx: old 0x%lx 0x%lx, new 0x%lx 0x%lx\n",
-              (ulong) pdref->value.pdict,
-              ((const ulong *)pkey)[0], ((const ulong *)pkey)[1],
-              (ulong) pvslot,
-              ((const ulong *)pvslot)[0], ((const ulong *)pvslot)[1],
-              ((const ulong *)pvalue)[0], ((const ulong *)pvalue)[1]);
+    if_debug8m('d', (const gs_memory_t *)mem,
+               "[d]0x%lx: put key 0x%lx 0x%lx\n  value at 0x%lx: old 0x%lx 0x%lx, new 0x%lx 0x%lx\n",
+               (ulong) pdref->value.pdict,
+               ((const ulong *)pkey)[0], ((const ulong *)pkey)[1],
+               (ulong) pvslot,
+               ((const ulong *)pvslot)[0], ((const ulong *)pvslot)[1],
+               ((const ulong *)pvalue)[0], ((const ulong *)pvalue)[1]);
     ref_assign_old_in(mem, &pdref->value.pdict->values, pvslot, pvalue,
                       "dict_put(value)");
     return rcode;
@@ -581,8 +582,9 @@ dict_undef(ref * pdref, const ref * pkey, dict_stack_t *pds)
         ref_packed *pkp = pdict->keys.value.writable_packed + index;
         bool must_save = ref_must_save_in(mem, &pdict->keys);
 
-        if_debug3('d', "[d]0x%lx: removing key at 0%lx: 0x%x\n",
-                  (ulong)pdict, (ulong)pkp, (uint)*pkp);
+        if_debug3m('d', (const gs_memory_t *)mem,
+                   "[d]0x%lx: removing key at 0%lx: 0x%x\n",
+                   (ulong)pdict, (ulong)pkp, (uint)*pkp);
         /* See the initial comment for why it is safe not to save */
         /* the change if the keys array itself is new. */
         if (must_save)
@@ -615,8 +617,9 @@ dict_undef(ref * pdref, const ref * pkey, dict_stack_t *pds)
     } else {			/* not packed */
         ref *kp = pdict->keys.value.refs + index;
 
-        if_debug4('d', "[d]0x%lx: removing key at 0%lx: 0x%lx 0x%lx\n",
-                  (ulong)pdict, (ulong)kp, ((ulong *)kp)[0], ((ulong *)kp)[1]);
+        if_debug4m('d', (const gs_memory_t *)mem,
+                   "[d]0x%lx: removing key at 0%lx: 0x%lx 0x%lx\n",
+                   (ulong)pdict, (ulong)kp, ((ulong *)kp)[0], ((ulong *)kp)[1]);
         make_null_old_in(mem, &pdict->keys, kp, "dict_undef(key)");
         /*
          * Accumulating deleted entries slows down lookup.
@@ -866,10 +869,10 @@ dict_next(const ref * pdref, int index, ref * eltp /* ref eltp[2] */ )
             (!dict_is_packed(pdict) && !r_has_type(eltp, t_null))
             ) {
             eltp[1] = *vp;
-            if_debug6('d', "[d]0x%lx: index %d: %lx %lx, %lx %lx\n",
-                      (ulong) pdict, index,
-                      ((ulong *) eltp)[0], ((ulong *) eltp)[1],
-                      ((ulong *) vp)[0], ((ulong *) vp)[1]);
+            if_debug6m('d', dict_mem(pdict), "[d]0x%lx: index %d: %lx %lx, %lx %lx\n",
+                       (ulong) pdict, index,
+                       ((ulong *) eltp)[0], ((ulong *) eltp)[1],
+                       ((ulong *) vp)[0], ((ulong *) vp)[1]);
             return index;
         }
     }
