@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Non-I/O file operators */
 #include "memory_.h"
 #include "string_.h"
@@ -663,8 +665,8 @@ ztempfile(i_ctx_t *i_ctx_p)
     const char *pstr;
     char fmode[4];
     int code = parse_file_access_string(op, fmode);
-    char prefix[gp_file_name_sizeof];
-    char fname[gp_file_name_sizeof];
+    char *prefix = NULL;
+    char *fname= NULL;
     uint fnlen;
     FILE *sfile;
     stream *s;
@@ -672,6 +674,13 @@ ztempfile(i_ctx_t *i_ctx_p)
 
     if (code < 0)
         return code;
+    prefix = (char *)gs_alloc_bytes(imemory, gp_file_name_sizeof, "ztempfile(prefix)");
+    fname = (char *)gs_alloc_bytes(imemory, gp_file_name_sizeof, "ztempfile(fname)");
+    if (!prefix || !fname) {
+        code = gs_note_error(gs_error_VMerror);
+        goto done;
+    }
+
     strcat(fmode, gp_fmode_binary_suffix);
     if (r_has_type(op - 1, t_null))
         pstr = gp_scratch_file_name_prefix;
@@ -680,8 +689,10 @@ ztempfile(i_ctx_t *i_ctx_p)
 
         check_read_type(op[-1], t_string);
         psize = r_size(op - 1);
-        if (psize >= gp_file_name_sizeof)
-            return_error(e_rangecheck);
+        if (psize >= gp_file_name_sizeof) {
+            code = gs_note_error(e_rangecheck);
+            goto done;
+        }
         memcpy(prefix, op[-1].value.const_bytes, psize);
         prefix[psize] = 0;
         pstr = prefix;
@@ -690,29 +701,37 @@ ztempfile(i_ctx_t *i_ctx_p)
     if (gp_file_name_is_absolute(pstr, strlen(pstr))) {
         if (check_file_permissions(i_ctx_p, pstr, strlen(pstr),
                                    "PermitFileWriting") < 0) {
-            return_error(e_invalidfileaccess);
+            code = gs_note_error(e_invalidfileaccess);
+            goto done;
         }
     } else if (!prefix_is_simple(pstr)) {
-        return_error(e_invalidfileaccess);
+        code = gs_note_error(e_invalidfileaccess);
+        goto done;
     }
 
     s = file_alloc_stream(imemory, "ztempfile(stream)");
-    if (s == 0)
-        return_error(e_VMerror);
+    if (s == 0) {
+        code = gs_note_error(e_VMerror);
+        goto done;
+    }
     buf = gs_alloc_bytes(imemory, file_default_buffer_size,
                          "ztempfile(buffer)");
-    if (buf == 0)
-        return_error(e_VMerror);
+    if (buf == 0) {
+        code = gs_note_error(e_VMerror);
+        goto done;
+    }
     sfile = gp_open_scratch_file(imemory, pstr, fname, fmode);
     if (sfile == 0) {
         gs_free_object(imemory, buf, "ztempfile(buffer)");
-        return_error(e_invalidfileaccess);
+        code = gs_note_error(e_invalidfileaccess);
+        goto done;
     }
     fnlen = strlen(fname);
     sbody = ialloc_string(fnlen, ".tempfile(fname)");
     if (sbody == 0) {
         gs_free_object(imemory, buf, "ztempfile(buffer)");
-        return_error(e_VMerror);
+        code = gs_note_error(e_VMerror);
+        goto done;
     }
     memcpy(sbody, fname, fnlen);
     file_init_stream(s, sfile, fmode, buf, file_default_buffer_size);
@@ -722,10 +741,17 @@ ztempfile(i_ctx_t *i_ctx_p)
         sclose(s);
         iodev_dflt->procs.delete_file(iodev_dflt, fname);
         ifree_string(sbody, fnlen, ".tempfile(fname)");
-        return_error(e_VMerror);
+        code = gs_note_error(e_VMerror);
+        goto done;
     }
     make_string(op - 1, a_readonly | icurrent_space, fnlen, sbody);
     make_stream_file(op, s, fmode);
+
+done:
+    if (prefix)
+        gs_free_object(imemory, prefix, "ztempfile(prefix)");
+    if (fname)
+        gs_free_object(imemory, fname, "ztempfile(fname)");
     return code;
 }
 
@@ -862,7 +888,7 @@ iodev_os_open_file(gx_io_device * iodev, const char *fname, uint len,
 {
     return file_open_stream(fname, len, file_access,
                             file_default_buffer_size, ps,
-                            iodev, iodev->procs.fopen, mem);
+                            iodev, iodev->procs.gp_fopen, mem);
 }
 
 /* Make a t_file reference to a stream. */
@@ -945,6 +971,8 @@ lib_file_open_search_with_combine(gs_file_path_ptr  lib_path, const gs_memory_t
             code = gs_parse_file_name(&pname, pstr, plen, mem);
             if (code < 0)
                 continue;
+            if (blen < max(pname.len, plen) + flen)
+            	return_error(e_limitcheck);
             memcpy(buffer, pname.fname, pname.len);
             memcpy(buffer+pname.len, fname, flen);
             code = pname.iodev->procs.open_file(pname.iodev, buffer, pname.len + flen, fmode,
@@ -996,6 +1024,11 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
     gs_main_instance *minst = get_minst_from_memory(mem);
     int code;
 
+    /* Once we've opened the arg file, prevent searching current dir from now on */
+    /* If this causes problems with "strange" scripts and utlities, we have to rethink */
+    if (i_ctx_p)
+        i_ctx_p->starting_arg_file = false;
+
     /* when starting arg files (@ files) iodev_default is not yet set */
     if (iodev == 0)
         iodev = (gx_io_device *)gx_io_device_table[0];
@@ -1086,7 +1119,7 @@ file_read_string(const byte *str, uint len, ref *pfile, gs_ref_memory_t *imem)
 int
 filter_report_error(stream_state * st, const char *str)
 {
-    if_debug1('s', "[s]stream error: %s\n", str);
+    if_debug1m('s', st->memory, "[s]stream error: %s\n", str);
     strncpy(st->error_string, str, STREAM_MAX_ERROR_STRING);
     /* Ensure null termination. */
     st->error_string[STREAM_MAX_ERROR_STRING] = 0;
